var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { Component, cloneElement } from "preact";
import * as querySelectorAll from "dom-helpers/query/querySelectorAll";
import { addClass, hasClass, removeClass } from "dom-helpers/class";
import * as ownerDocument from 'dom-helpers/ownerDocument';
import * as style from "dom-helpers/style";
import * as offset from "dom-helpers/query/offset";
import EventListener from '../vendor/EventListener';
import getEventClientPos from "../vendor/getPtClientPos";
import UserAgent from "../vendor/UserAgent_DEPRECATED";
import { throttle } from "throttle-debounce";
import bind from "../vendor/bind";
function callIfExists(func) {
    var args = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        args[_i - 1] = arguments[_i];
    }
    return (typeof func === 'function') && func.apply(void 0, args);
}
var g_lastMenu;
var ContextMenuTrigger = /** @class */ (function (_super) {
    __extends(ContextMenuTrigger, _super);
    function ContextMenuTrigger(props) {
        var _this = _super.call(this, props) || this;
        _this.elemRef = function (c) {
            _this.elem = c;
        };
        _this.updatePosition = throttle(16, _this.updatePosition.bind(_this));
        _this._childrenAttributes = props.children[0].attributes;
        return _this;
    }
    ContextMenuTrigger.prototype.componentDidMount = function () {
        this._eventResizeToken = EventListener.listen(window, "resize", this._onResize);
        this._eventKeyupToken = EventListener.listen(document.documentElement, "keyup", this._onKeyUp);
        this._eventClickToken = EventListener.listen(document.documentElement, "click", this._onClick);
        this._eventContextMenuToken = EventListener.listen(document.documentElement, "contextmenu", this._onClick);
    };
    ContextMenuTrigger.prototype.componentWillUnmount = function () {
        if (this._eventResizeToken) {
            this._eventResizeToken.remove();
            this._eventResizeToken = null;
        }
        if (this._eventKeyupToken) {
            this._eventKeyupToken.remove();
            this._eventKeyupToken = null;
        }
        if (this._eventClickToken) {
            this._eventClickToken.remove();
            this._eventClickToken = null;
        }
        if (this._eventContextMenuToken) {
            this._eventContextMenuToken.remove();
            this._eventContextMenuToken = null;
        }
    };
    ContextMenuTrigger.prototype._onResize = function () {
        this.updatePosition(this._x, this._y);
    };
    ContextMenuTrigger.prototype._onKeyUp = function (event) {
        if (event.keyCode == 27) {
            this.closeLastMenu();
        }
    };
    ContextMenuTrigger.prototype._onClick = function () {
        this.closeLastMenu();
    };
    ContextMenuTrigger.prototype.handleMouseDown = function (event) {
        callIfExists(this._childrenAttributes.onMouseDown, event);
    };
    ContextMenuTrigger.prototype.handleMouseUp = function (event) {
        if (event.button === 0) {
            clearTimeout(this.mouseDownTimeoutId);
        }
        callIfExists(this._childrenAttributes.onMouseUp, event);
    };
    ContextMenuTrigger.prototype.handleMouseOut = function (event) {
        if (event.button === 0) {
            clearTimeout(this.mouseDownTimeoutId);
        }
        callIfExists(this._childrenAttributes.onMouseOut, event);
    };
    ContextMenuTrigger.prototype.handleTouchStart = function (event) {
        var _this = this;
        this.touchHandled = false;
        if (this.props.holdToDisplay >= 0 && event.touches.length == 1 && UserAgent.mobile()) {
            this.closeLastMenu();
            this.touchstartTimeoutId && clearTimeout(this.touchstartTimeoutId);
            this.touchstartTimeoutId = setTimeout(function () {
                event.touches.length == 1 && _this.handleContextClick(event);
                _this.touchHandled = true;
            }, this.props.holdToDisplay);
        }
        else if (event.touches.length > 1) {
            this.closeLastMenu();
        }
        callIfExists(this._childrenAttributes.onTouchStart, event);
        var _a = getEventClientPos(event), clientX = _a.clientX, clientY = _a.clientY;
        this._lastX = clientX;
        this._lastY = clientY;
    };
    ContextMenuTrigger.prototype.handleTouchMove = function (event) {
        var _a = getEventClientPos(event), clientX = _a.clientX, clientY = _a.clientY;
        if (Math.abs(this._lastX - clientX) > 3 || Math.abs(this._lastY - clientY) > 3) {
            this.touchstartTimeoutId && clearTimeout(this.touchstartTimeoutId) && this.touchstartTimeoutId == null;
            this.closeLastMenu();
        }
    };
    ContextMenuTrigger.prototype.handleTouchEnd = function (event) {
        if (this.touchHandled) {
            event.preventDefault();
        }
        clearTimeout(this.touchstartTimeoutId);
        callIfExists(this._childrenAttributes.onTouchEnd, event);
    };
    ContextMenuTrigger.prototype.handleContextMenu = function (event) {
        this.handleContextClick(event);
        callIfExists(this._childrenAttributes.onContextMenu, event);
    };
    ContextMenuTrigger.prototype.getMenu = function () {
        var selector = this.props.target;
        return querySelectorAll(document.body, selector)[0];
    };
    ContextMenuTrigger.prototype.getContainer = function (container, defaultContainer) {
        container = typeof container === 'function' ? container() : container;
        if (!container) {
            return defaultContainer;
        }
        return container.base || defaultContainer;
    };
    ContextMenuTrigger.prototype.updatePosition = function (pageX, pageY) {
        // window height change then adjust menuElement properties
        // only adjust the top height overflow-y
        var menu = this.getMenu();
        if (hasClass(menu, "open")) {
            var container = this.getContainer(this.props.container, ownerDocument(this.base).body);
            var docElem = document.documentElement;
            var scrollTop = window.pageYOffset //用于FF
                || docElem.scrollTop;
            var scrollLeft = window.pageXOffset //用于FF
                || docElem.scrollLeft;
            var docLeft = scrollLeft - (docElem.clientLeft || 0);
            var docTop = scrollTop - (docElem.clientTop || 0);
            var left = Math.max(pageX - docLeft, 0);
            var top_1 = Math.max(pageY - docTop, 0);
            // const docHeight = docElem.clientHeight + docTop;
            var menuHeight = menu.scrollHeight || 0;
            var menuWidth = menu.scrollWidth || 0;
            var containerOffset = offset(container);
            var containerHeight = containerOffset.height - 2;
            if (menuHeight >= containerHeight) {
                top_1 = containerOffset.top;
            }
            else if (top_1 + menuHeight > containerOffset.top + containerOffset.height) {
                top_1 = containerOffset.top + containerOffset.height - menuHeight;
            }
            if (left + menuWidth > containerOffset.left + containerOffset.width) {
                left = containerOffset.left + containerOffset.width - menuWidth;
            }
            style(querySelectorAll(menu, ".dropdown-menu")[0], { "max-height": containerHeight + 'px', "overflow-y": "auto" });
            style(menu, { 'top': top_1 + 'px', left: left + 'px' });
        }
    };
    ContextMenuTrigger.prototype.showMenu = function (event) {
        var x = event.clientX || (event.touches && event.touches[0].pageX);
        var y = event.clientY || (event.touches && event.touches[0].pageY);
        this._x = x;
        this._y = y;
        var menu = this.getMenu();
        addClass(menu, "open");
        g_lastMenu = menu;
        if (this.props.showTop) {
            var pos = offset(this.base);
            y = pos.top - offset(menu).height;
            x = pos.left;
        }
        this.updatePosition(x, y);
    };
    ContextMenuTrigger.prototype.closeLastMenu = function () {
        if (g_lastMenu) {
            removeClass(g_lastMenu, "open");
            g_lastMenu = null;
        }
    };
    ContextMenuTrigger.prototype.handleContextClick = function (event) {
        if (this.props.disable)
            return;
        event.preventDefault();
        event.stopPropagation();
        this.closeLastMenu();
        this.showMenu(event);
    };
    ContextMenuTrigger.prototype.render = function () {
        var newAttrs = Object.assign({
            onContextMenu: this.handleContextMenu,
            onMouseDown: this.handleMouseDown,
            onMouseUp: this.handleMouseUp,
            onTouchStart: this.handleTouchStart,
            onTouchEnd: this.handleTouchEnd,
            onTouchMove: this.handleTouchMove,
            onMouseOut: this.handleMouseOut,
            ref: this.elemRef
        });
        return this.props.children.map(function (element) {
            return cloneElement(element, newAttrs);
        })[0];
    };
    __decorate([
        bind
    ], ContextMenuTrigger.prototype, "_onResize", null);
    __decorate([
        bind
    ], ContextMenuTrigger.prototype, "_onKeyUp", null);
    __decorate([
        bind
    ], ContextMenuTrigger.prototype, "_onClick", null);
    __decorate([
        bind
    ], ContextMenuTrigger.prototype, "handleMouseDown", null);
    __decorate([
        bind
    ], ContextMenuTrigger.prototype, "handleMouseUp", null);
    __decorate([
        bind
    ], ContextMenuTrigger.prototype, "handleMouseOut", null);
    __decorate([
        bind
    ], ContextMenuTrigger.prototype, "handleTouchStart", null);
    __decorate([
        bind
    ], ContextMenuTrigger.prototype, "handleTouchMove", null);
    __decorate([
        bind
    ], ContextMenuTrigger.prototype, "handleTouchEnd", null);
    __decorate([
        bind
    ], ContextMenuTrigger.prototype, "handleContextMenu", null);
    __decorate([
        bind
    ], ContextMenuTrigger.prototype, "handleContextClick", null);
    return ContextMenuTrigger;
}(Component));
export default ContextMenuTrigger;
//# sourceMappingURL=ContextMenuTrigger.js.map