var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { h, Component } from "preact";
import * as query from "dom-helpers/query";
import DocViewScrollHelper from "./DocViewScrollHelper";
import translateDOMPositionXY from "../vendor/translateDOMPositionXY";
import DocViewSizeHelper, { PAGE_PADDING } from './DocViewSizeHelper';
import WheelHandler from "./WheelHandler";
import DocViewBufferedRows from "./DocViewBufferedRows";
import { getWheelEventType } from "../vendor/normalizeWheel";
import DOMMouseMoveTracker from "../vendor/DOMMouseMoveTracker";
import Scrollbar from "./Scrollbar";
import Rect from "./Rect";
import UserAgent from "../vendor/UserAgent_DEPRECATED";
import clamp from "../vendor/clamp";
import { throttle } from "throttle-debounce";
import EventListener from '../vendor/EventListener';
import { getPixelsPerPoint } from "../utils/common";
import AnimateExecutor from "../vendor/AnimateExecutor";
import Spring from "../vendor/Spring";
import Point from "../utils/point";
import bind from "../vendor/bind";
import isTouchEvent from "../vendor/isTouchEvent";
import * as cuid from "cuid";
import Hammer from "hammerjs";
require('../style/cpdf.less');
var DocView = /** @class */ (function (_super) {
    __extends(DocView, _super);
    function DocView(props) {
        var _this = _super.call(this, props) || this;
        _this._docViewId = cuid();
        _this._sizeHelper = new DocViewSizeHelper(props.manifestInfo);
        _this._onResize = throttle(50, _this._onResize.bind(_this));
        _this._onPageIndexChange = _this._onPageIndexChange.bind(_this);
        _this._onZoomChange = _this._onZoomChange.bind(_this);
        _this._animateExecutor = new AnimateExecutor();
        var defaultRowHeight = _this._sizeHelper.getDefaultRowHeight(DocView.defaultState.zoom);
        var viewportHeight = DocView.defaultState.height;
        _this._scrollHelper = new DocViewScrollHelper(props.pageCount, defaultRowHeight, viewportHeight, _this._getRowHeight(props.zoom));
        _this.state = Object.assign({}, DocView.defaultState, { zoom: props.zoom, pageIndex: props.pageIndex });
        return _this;
    }
    DocView.prototype.getChildContext = function () {
        return {
            sizeHelper: this._sizeHelper,
            manifestInfo: this.props.manifestInfo,
            userInfo: this.props.userInfo,
            docId: this.props.docId,
            serverUrl: this.props.serverUrl,
            pagePtToDoc: this.pagePtToDoc,
            getPageByDocPt: this.getPageByDocPt,
            docPtToPage: this.docPtToPage,
            ptToPDF: this.ptToPDF,
            pdfToPage: this.pdfToPage,
            rectToPage: this.rectToPage,
            lengthToPDF: this.lengthToPDF,
            lengthToPage: this.lengthToPage,
            scrollTo: this.scrollTo,
            getPageWidth: this.getPageWidth,
            docViewId: this._docViewId,
            autoParse: this.props.autoParse,
            imageWithAnnot: this.props.imageWithAnnot,
            imageWithForm: this.props.imageWithForm
        };
    };
    DocView.prototype.scrollToAnimate = function (scrollY) {
        var _this = this;
        return new Promise(function (resolve) {
            var distance = scrollY - _this.state.scrollY;
            var originY = _this.state.scrollY;
            if (distance != 0) {
                _this._animateExecutor.execute(distance, function (delta) {
                    _this._onVerticalScroll(originY + delta);
                }, function () {
                    resolve(_this.state.scrollY);
                });
            }
            else {
                resolve(_this.state.scrollY);
            }
        });
    };
    DocView.prototype.scrollTo = function (ptDoc) {
        return this.scrollToAnimate(ptDoc.y - this.state.height / 2);
    };
    DocView.prototype.getPageWidth = function () {
        return this._getRowWidth(this.state.zoom)(0);
    };
    DocView.prototype.pagePtToDoc = function (pageIndex, pt) {
        return {
            x: pt.x,
            y: pt.y + this._getRowPosition(pageIndex)
        };
    };
    DocView.prototype.docPtToPage = function (pageIndex, pt) {
        return {
            x: pt.x,
            y: pt.y - this._getRowPosition(pageIndex)
        };
    };
    DocView.prototype.lengthToPDF = function (length) {
        return length / (getPixelsPerPoint() * this.state.zoom);
    };
    DocView.prototype.lengthToPage = function (length) {
        return length * (getPixelsPerPoint() * this.state.zoom);
    };
    DocView.prototype.ptToPDF = function (pageIndex, pt) {
        var x = pt.x / (getPixelsPerPoint() * this.state.zoom);
        var y = pt.y / (getPixelsPerPoint() * this.state.zoom);
        var _a = this.getPageSize(pageIndex), width = _a.width, height = _a.height;
        return new Point(x, height - y);
    };
    DocView.prototype.pdfToPage = function (pageIndex, pt) {
        var _a = this.getPageSize(pageIndex), width = _a.width, height = _a.height;
        var x = pt.x * getPixelsPerPoint() * this.state.zoom;
        var y = (height - pt.y) * getPixelsPerPoint() * this.state.zoom;
        return new Point(x, y);
    };
    DocView.prototype.getPageSize = function (pageIndex) {
        var manifestInfo = this.props.manifestInfo;
        var pagesInfo = manifestInfo.PagesInfo;
        if (pageIndex >= 0 && pageIndex < pagesInfo.length) {
            return {
                width: pagesInfo[pageIndex].width,
                height: pagesInfo[pageIndex].height
            };
        }
        else {
            return {
                width: 0,
                height: 0
            };
        }
    };
    DocView.prototype.rectToPage = function (pageIndex, rect) {
        var zoom = this.state.zoom;
        var pageSize = this.getPageSize(pageIndex);
        return {
            bottom: (pageSize.height - rect.bottom) * getPixelsPerPoint() * zoom,
            top: (pageSize.height - rect.top) * getPixelsPerPoint() * zoom,
            left: rect.left * getPixelsPerPoint() * zoom,
            right: rect.right * getPixelsPerPoint() * zoom
        };
    };
    DocView.prototype.getPageByDocPt = function (pt) {
        for (var i = this.state.firstRowIndex; i < this.props.pageCount; ++i) {
            if (pt.y >= this._getRowPosition(i) - PAGE_PADDING / 2 &&
                pt.y <= this._getRowPosition(i) + this._getRowHeight(this.state.zoom)(i) - PAGE_PADDING / 2) {
                return i;
            }
        }
    };
    DocView.prototype.componentDidMount = function () {
        var _this = this;
        this._isMounted = true;
        this._wheelHandler = new WheelHandler(this._onWheel, this._shouldHandleWheelX, this._shouldHandleWheelY);
        this._mouseMoveTracker = new DOMMouseMoveTracker(this._onMouseMove, this._onMouseMoveEnd, document.documentElement);
        this._eventDownToken = EventListener.listen(this.base, 'mousedown', this._onMouseDown_TouchStart);
        this._eventTouchStartToken = EventListener.listen(this.base, 'touchstart', this._onMouseDown_TouchStart);
        this._eventWheelToken = EventListener.listen(this.base, getWheelEventType(), this._wheelHandler.onWheel);
        this._eventOrientationToken = EventListener.listen(window, "orientationchange", this._onOrientationChange);
        this._eventResizeToken = EventListener.listen(window, "resize", this._onResize);
        this._eventTouchStartDocumentToken = EventListener.listen(document.documentElement, "touchstart", function (event) {
            if (event.touches.length > 1) {
                event.preventDefault();
            }
        });
        var lastTouchEnd = 0;
        this._eventTouchEndDocumentToken = EventListener.listen(document.documentElement, 'touchend', function (event) {
            var now = Date.now();
            if (now - lastTouchEnd <= 300) {
                event.preventDefault();
            }
            lastTouchEnd = now;
        });
        this.context.webpdf.on("RESIZE", this._onResize);
        setTimeout(function () {
            var docViewWidth = query.width(_this.base.parentNode, false);
            var docViewHeight = query.height(_this.base.parentNode, false);
            var _a = _this.autoFitWidth(docViewWidth), zoom = _a.zoom, minZoom = _a.minZoom;
            console.log("autoFit", zoom, minZoom);
            _this.setState(_this._calculateState({ zoom: zoom, minZoom: minZoom, width: docViewWidth, height: docViewHeight }));
        }, 100);
        this.bindMobile();
    };
    DocView.prototype.bindMobile = function () {
        var _this = this;
        if (UserAgent.mobile()) {
            var scrollStop;
            var distance;
            var spring;
            var curPos;
            var animationFrameID = null;
            var zoomStart;
            var center;
            var scale;
            var mc = new Hammer.Manager(this.base);
            var pinch = new window.Hammer.Pinch({ threshold: 0.03 });
            var swipe = new window.Hammer.Swipe({ direction: window.Hammer.DIRECTION_VERTICAL });
            var tap = new window.Hammer.Tap({ event: 'singletap' });
            mc.add([pinch, swipe, tap]);
            var step = function () {
                animationFrameID = null;
                var p = spring.step();
                _this.wheel((p - curPos) * distance / 100);
                curPos = p;
                if (spring.moving && !scrollStop) {
                    animationFrameID = window.requestAnimationFrame(step);
                }
            };
            var zoomByPinch = function () {
                animationFrameID = null;
                _this.zoomByPinch(scale * zoomStart, new Point(center.x, center.y).ScreenToElement(_this.base));
            };
            mc.on("singletap", function () {
                scrollStop = true;
            });
            mc.on("swipedown", function (event) {
                console.log("swipedown");
                if (Math.abs(event.velocity) < 0.10)
                    return;
                distance = -(event.distance + event.distance * Math.abs(event.velocity));
                spring = new Spring({ velocity: Math.abs(event.velocity), tension: 300, friction: 80 });
                curPos = 0;
                scrollStop = false;
                if (animationFrameID == null) {
                    step();
                }
            });
            mc.on("swipeup", function (event) {
                if (Math.abs(event.velocity) < 0.10)
                    return;
                distance = (event.distance + event.distance * Math.abs(event.velocity));
                spring = new Spring({ velocity: event.velocity, tension: 300, friction: 80 });
                curPos = 0;
                scrollStop = false;
                if (animationFrameID == null) {
                    step();
                }
            });
            mc.on("pinchstart", function (event) {
                zoomStart = _this.state.zoom;
                center = event.center;
            });
            mc.on("pinchin", function (event) {
                console.log("pinchin");
                if (animationFrameID === null) {
                    scale = event.scale;
                    center = event.center;
                    animationFrameID = requestAnimationFrame(zoomByPinch);
                }
                event.preventDefault();
            });
            mc.on("pinchout", function (event) {
                console.log("pinchout");
                if (animationFrameID === null) {
                    scale = event.scale;
                    center = event.center;
                    animationFrameID = requestAnimationFrame(zoomByPinch);
                }
                event.preventDefault();
            });
        }
    };
    DocView.prototype.autoFitWidth = function (docViewWidth) {
        if (UserAgent.mobile()) {
            var pageWidth = this._sizeHelper.getMaxPageWidth(1);
            var minZoom = docViewWidth / pageWidth;
            return { zoom: minZoom, minZoom: minZoom };
        }
        else {
            return this.state;
        }
    };
    DocView.prototype.componentWillUnmount = function () {
        this._isMounted = false;
        if (this._eventDownToken) {
            this._eventDownToken.remove();
            this._eventDownToken = null;
        }
        if (this._eventTouchStartToken) {
            this._eventTouchStartToken.remove();
            this._eventTouchStartToken = null;
        }
        if (this._eventWheelToken) {
            this._eventWheelToken.remove();
            this._eventWheelToken = null;
        }
        if (this._eventResizeToken) {
            this._eventResizeToken.remove();
            this._eventResizeToken = null;
        }
        if (this._eventOrientationToken) {
            this._eventOrientationToken.remove();
            this._eventOrientationToken = null;
        }
        if (this._eventTouchEndDocumentToken) {
            this._eventTouchEndDocumentToken.remove();
            this._eventTouchEndDocumentToken = null;
        }
        if (this._eventTouchStartDocumentToken) {
            this._eventTouchStartDocumentToken.remove();
            this._eventTouchStartDocumentToken = null;
        }
        this.context.webpdf.removeListener("RESIZE", this._onResize);
    };
    DocView.prototype._onOrientationChange = function () {
        console.log("_onOrientationChange");
        this._onResize();
    };
    DocView.prototype.pageIndexChange = function (pageIndex) {
        var pageCount = this.props.pageCount;
        return pageIndex >= 0 && pageIndex <= pageCount - 1 && pageIndex != this.state.pageIndex;
    };
    DocView.prototype.componentWillReceiveProps = function (nextProps) {
        var pageIndex = nextProps.pageIndex;
        if (this.pageIndexChange(pageIndex)) {
            var scrollState = this._scrollHelper.scrollToRow(pageIndex, 0);
            this.setState(this._calculateState({
                firstRowIndex: scrollState.index,
                firstRowOffset: scrollState.offset,
                scrollY: scrollState.position,
                pageIndex: pageIndex,
                zoom: nextProps.zoom
            }));
        }
        else {
            this.setState(this._calculateState({ zoom: nextProps.zoom }));
        }
    };
    DocView.prototype.wheel = function (deltaY) {
        if (this._shouldHandleWheelY(deltaY)) {
            var scrollState = this._scrollHelper.scrollBy(deltaY);
            this.setState(this._calculateState({
                firstRowIndex: scrollState.index,
                firstRowOffset: scrollState.offset,
                scrollY: scrollState.position,
                scrollContentHeight: scrollState.contentHeight,
            }));
        }
    };
    DocView.prototype.zoomByPinch = function (after, center) {
        var state = this.state;
        var afterZoom = clamp(after, state.minZoom, 2.0);
        if (afterZoom == state.zoom)
            return;
        console.log("center", afterZoom, center);
        //before
        var currentRowWidth = this._sizeHelper.getMaxPageWidth(state.zoom);
        var x = this.state.scrollX + center.x - Math.max(0, (state.width - currentRowWidth) / 2);
        var y = this.state.scrollY + center.y;
        var currentPageIndex = state.firstRowIndex;
        var offsetYToPage = y - this._scrollHelper.getRowPosition(currentPageIndex);
        var offsetXToPage = x;
        var rate = afterZoom / state.zoom;
        console.log(offsetXToPage, offsetYToPage);
        //this.setZoomPrepare(after);
        //after   
        var _scrollHelper = new DocViewScrollHelper(this.props.pageCount, this._sizeHelper.getDefaultRowHeight(afterZoom), state.height, this._getRowHeight(afterZoom));
        var position = this._scrollHelper.getRowPosition(currentPageIndex);
        offsetXToPage *= rate;
        offsetYToPage *= rate;
        console.log(offsetXToPage, offsetYToPage);
        var x1 = offsetXToPage;
        var y1 = offsetYToPage + position;
        var scrollX = Math.max(0, x1 - center.x);
        var scrollY = y1 - center.y;
        var scrollContentWidth = this._sizeHelper.getMaxPageWidth(afterZoom);
        var scrollState = _scrollHelper.scrollTo(scrollY);
        this.setState({ zoom: afterZoom });
        this.setState(this._calculateState({
            firstRowIndex: scrollState.index,
            firstRowOffset: scrollState.offset,
            scrollY: scrollState.position,
            scrollX: scrollX,
            scrollContentHeight: scrollState.contentHeight,
            scrollContentWidth: scrollContentWidth,
            maxScrollX: Math.max(0, scrollContentWidth - state.width),
            maxScrollY: Math.max(0, scrollState.contentHeight - state.height),
            zoom: afterZoom
        }));
    };
    DocView.prototype._onResize = function () {
        if (this._isMounted) {
            var docViewWidth = query.width(this.base.parentNode, false);
            var docViewHeight = query.height(this.base.parentNode, false);
            var minZoom = this.autoFitWidth(docViewWidth).minZoom;
            this.setState(this._calculateState({ width: docViewWidth, height: docViewHeight, minZoom: minZoom }));
        }
    };
    DocView.prototype._onWheel = function (deltaX, deltaY) {
        if (!this.state.isScrolling) {
            this._didScrollStart();
        }
        if (Math.abs(deltaY) > Math.abs(deltaX)) {
            var scrollState = this._scrollHelper.scrollBy(deltaY);
            this.setState(this._calculateState({
                firstRowIndex: scrollState.index,
                firstRowOffset: scrollState.offset,
                scrollY: scrollState.position,
                scrollContentHeight: scrollState.contentHeight,
            }));
        }
        this._didScrollStop();
    };
    DocView.prototype._onVerticalScroll = function (scrollPos) {
        if (this._isMounted && scrollPos !== this.state.scrollY) {
            if (!this.state.isScrolling) {
                this._didScrollStart();
            }
            var scrollState = this._scrollHelper.scrollTo(scrollPos);
            this.setState(this._calculateState({
                scrollY: scrollState.position,
                scrollContentHeight: scrollState.contentHeight,
                firstRowOffset: scrollState.offset,
                firstRowIndex: scrollState.index
            }));
            this._didScrollStop();
        }
    };
    DocView.prototype._onHorizontalScroll = function (scrollPos) {
        if (this._isMounted && scrollPos !== this.state.scrollX) {
            if (!this.state.isScrolling) {
                this._didScrollStart();
            }
            this.setState({
                scrollX: scrollPos,
            });
            this._didScrollStop();
        }
    };
    DocView.prototype._shouldHandleWheelX = function (delta) {
        if (delta === 0) {
            return false;
        }
        delta = Math.round(delta);
        if (delta === 0) {
            return false;
        }
        return ((delta < 0 && this.state.scrollX > 0) ||
            (delta >= 0 && this.state.scrollX < this.state.maxScrollX));
    };
    DocView.prototype._shouldHandleWheelY = function (delta) {
        if (delta === 0) {
            return false;
        }
        delta = Math.round(delta);
        if (delta === 0) {
            return false;
        }
        return ((delta < 0 && this.state.scrollY > 0) ||
            (delta >= 0 && this.state.scrollY < this.state.maxScrollY));
    };
    DocView.prototype._renderRows = function () {
        var state = this.state;
        var props = this.props;
        if (state.width == 100) {
            return h("div", null);
        }
        else {
            return (h(DocViewBufferedRows, { rowsCount: props.pageCount, defaultRowHeight: this._sizeHelper.getDefaultRowHeight(state.zoom), width: state.width, height: state.height, zoom: state.zoom, scrollY: state.scrollY, visibleDocRect: state.visibleDocRect, isScrolling: state.isScrolling, firstRowIndex: state.firstRowIndex, firstRowOffset: state.firstRowOffset, rowHeightGetter: this._getRowHeight(state.zoom), rowWidthGetter: this._getRowWidth(state.zoom), rowPositionGetter: this._getRowPosition }));
        }
    };
    DocView.prototype.render = function (props, state) {
        var style = {
            position: 'absolute',
            top: 0,
            left: 0
        };
        var showScrollbarX = state.maxScrollX > 0;
        var showScrollbarY = state.maxScrollY > 0;
        var scrollbarXHeight = showScrollbarX ? Scrollbar.SIZE : 0;
        var scrollbarYHeight = state.height - scrollbarXHeight;
        translateDOMPositionXY(style, -state.scrollX, -state.scrollY);
        var verticalScrollbar;
        if (showScrollbarY) {
            verticalScrollbar =
                h(Scrollbar, { size: scrollbarYHeight, contentSize: scrollbarYHeight + state.maxScrollY, onScroll: this._onVerticalScroll, verticalTop: 0, position: state.scrollY });
        }
        var horizontalScrollbar;
        if (showScrollbarX) {
            var scrollbarXWidth = state.width;
            horizontalScrollbar =
                h(HorizontalScrollbar, { contentSize: scrollbarXWidth + state.maxScrollX, offset: 0, onScroll: this._onHorizontalScroll, position: state.scrollX, size: scrollbarXWidth });
        }
        var rows = this._renderRows();
        return (h("div", { class: "docViewer", id: this._docViewId },
            h("div", { class: "pageWrapper", style: style }, rows),
            verticalScrollbar,
            horizontalScrollbar,
            this.context.plugins.map(function (plugin) {
                return plugin.renderDoc && plugin.renderDoc({
                    zoom: state.zoom,
                    scrollY: state.scrollY,
                    pageIndex: state.pageIndex,
                    pageCount: props.pageCount,
                    docViewWidth: state.width,
                    docViewHeight: state.height,
                    serverUrl: props.serverUrl,
                    docId: props.docId
                });
            })));
    };
    DocView.prototype._onPageIndexChange = function (pageIndex) {
        if (this.props.pageIndex != pageIndex) {
            this.props.setPageIndex(pageIndex);
        }
    };
    DocView.prototype._onZoomChange = function (zoom) {
        if (this.props.zoom != zoom) {
            this.props.setZoom(zoom);
        }
    };
    DocView.prototype._didScrollStart = function () {
        if (this._isMounted && !this.state.isScrolling) {
            this.setState({ isScrolling: true });
        }
    };
    DocView.prototype._didScrollStop = function () {
        if (this._isMounted && this.state.isScrolling) {
            this.setState({ isScrolling: false });
        }
    };
    DocView.prototype._calcVisibleDocRect = function (_a, oldState) {
        var firstRowIndex = _a.firstRowIndex, firstRowOffset = _a.firstRowOffset, scrollX = _a.scrollX, width = _a.width, height = _a.height, scrollY = _a.scrollY, zoom = _a.zoom;
        var firstRowPosition = this._scrollHelper.getRowPosition(firstRowIndex);
        var visibleDocRect = new Rect({
            top: -(firstRowOffset - firstRowPosition),
            left: scrollX,
            width: width,
            height: height
        });
        var extendRate = zoom * 5;
        if (scrollY > oldState.scrollY) {
            visibleDocRect.height += (scrollY - oldState.scrollY) * extendRate;
        }
        else if (scrollY < oldState.scrollY) {
            visibleDocRect.top += (scrollY - oldState.scrollY) * extendRate;
            visibleDocRect.height += Math.abs(scrollY - oldState.scrollY) * extendRate;
        }
        if (scrollX > oldState.scrollX) {
            visibleDocRect.width += (scrollX - oldState.scrollX) * extendRate;
        }
        else if (scrollX > oldState.scrollX) {
            visibleDocRect.left += (scrollX - oldState.scrollX) * extendRate;
            visibleDocRect.width += Math.abs(scrollX - oldState.scrollX) * extendRate;
        }
        return visibleDocRect;
    };
    DocView.prototype._calculateState = function (nextState) {
        var props = this.props;
        var oldState = this.state;
        nextState = Object.assign({}, oldState, nextState);
        var width = nextState.width, height = nextState.height;
        if (width != oldState.width || height != oldState.height) {
            this.props.setDocViewSize(width, height);
        }
        var scrollX = nextState.scrollX;
        var scrollY = nextState.scrollY;
        var firstRowOffset = nextState.firstRowOffset;
        var zoom = Math.max(nextState.minZoom, nextState.zoom);
        if (zoom != oldState.zoom && oldState.zoom) {
            var scale = zoom / oldState.zoom;
            firstRowOffset = firstRowOffset * scale;
            scrollX = scrollX * scale;
        }
        var scrollContentHeight = nextState.scrollContentHeight, scrollContentWidth = nextState.scrollContentWidth, maxScrollX = nextState.maxScrollX, maxScrollY = nextState.maxScrollY, isScrolling = nextState.isScrolling, firstRowIndex = nextState.firstRowIndex, minZoom = nextState.minZoom;
        if (zoom != oldState.zoom || height != oldState.height || width != oldState.width) {
            var defaultRowHeight = this._sizeHelper.getDefaultRowHeight(zoom);
            this._scrollHelper = new DocViewScrollHelper(props.pageCount, defaultRowHeight, height, this._getRowHeight(zoom));
            var scrollState = this._scrollHelper.scrollToRow(firstRowIndex, firstRowOffset);
            scrollContentWidth = this._sizeHelper.getMaxPageWidth(zoom);
            scrollContentHeight = this._scrollHelper.getContentHeight();
            maxScrollX = Math.max(0, scrollContentWidth - width);
            maxScrollY = Math.max(0, scrollContentHeight - height);
            scrollX = Math.min(scrollX, maxScrollX);
            scrollY = Math.min(scrollState.position, maxScrollY);
        }
        var pageIndex = firstRowIndex;
        var pageAboveHeight = this._sizeHelper.getPageSize(firstRowIndex, zoom).height - (scrollY - this._scrollHelper.getRowPosition(firstRowIndex));
        if (pageAboveHeight < height / 2 - 20) {
            pageIndex = firstRowIndex + 1;
        }
        if (pageIndex !== undefined && pageIndex !== oldState.pageIndex && pageIndex !== this.props.pageIndex) {
            this._onPageIndexChange(pageIndex);
        }
        if (zoom != oldState.zoom) {
            this._onZoomChange(zoom);
        }
        var newState = {
            firstRowIndex: firstRowIndex,
            firstRowOffset: firstRowOffset,
            scrollX: scrollX,
            scrollY: scrollY,
            scrollContentHeight: scrollContentHeight,
            scrollContentWidth: scrollContentWidth,
            zoom: zoom,
            width: width,
            height: height,
            isScrolling: isScrolling,
            maxScrollY: maxScrollY,
            maxScrollX: maxScrollX,
            pageIndex: pageIndex,
            minZoom: minZoom
        };
        return Object.assign(newState, {
            visibleDocRect: this._calcVisibleDocRect(newState, oldState)
        });
    };
    DocView.prototype._getRowPosition = function (index) {
        return this._scrollHelper.getRowPosition(index);
    };
    DocView.prototype._getRowHeight = function (zoom) {
        var _this = this;
        return function (index) {
            return _this._sizeHelper.getPageSize(index, zoom).height;
        };
    };
    DocView.prototype._getRowWidth = function (zoom) {
        var _this = this;
        return function (index) {
            return _this._sizeHelper.getPageSize(index, zoom).width;
        };
    };
    DocView.prototype._onMouseDown_TouchStart = function (event) {
        console.log("start");
        if (isTouchEvent(event)) {
            if (event.touches.length > 1) {
                return;
            }
        }
        this._mouseMoveTracker.captureMouseMoves(event);
        this.base.focus();
    };
    DocView.prototype._onMouseMoveEnd = function () {
        this._mouseMoveTracker.releaseMouseMoves();
    };
    DocView.prototype._onMouseMove = function (deltaX, deltaY) {
        if (!this.state.isScrolling) {
            this._didScrollStart();
        }
        var handlerWheelX = this._shouldHandleWheelX(-deltaX);
        var handlerWheelY = this._shouldHandleWheelY(-deltaY);
        var scrollX = this.state;
        var newState = {};
        if (handlerWheelX) {
            newState.scrollX = this.state.scrollX - deltaX;
        }
        if (handlerWheelY) {
            var scrollState = this._scrollHelper.scrollBy(Math.round(-deltaY));
            newState.firstRowIndex = scrollState.index;
            newState.firstRowOffset = scrollState.offset;
            newState.scrollY = scrollState.position;
            newState.scrollContentHeight = scrollState.contentHeight;
        }
        this.setState(this._calculateState(newState));
        this._didScrollStop();
    };
    DocView.defaultState = {
        width: 100,
        height: 100,
        firstRowIndex: 0,
        firstRowOffset: 0,
        maxScrollY: 0,
        maxScrollX: 0,
        scrollX: 0,
        scrollY: 0,
        zoom: 1,
        scrollContentHeight: 0,
        scrollContentWidth: 0,
        isScrolling: false,
        pageIndex: 0,
        minZoom: 0.01,
        visibleDocRect: Rect.emptyRect
    };
    __decorate([
        bind
    ], DocView.prototype, "scrollTo", null);
    __decorate([
        bind
    ], DocView.prototype, "getPageWidth", null);
    __decorate([
        bind
    ], DocView.prototype, "pagePtToDoc", null);
    __decorate([
        bind
    ], DocView.prototype, "docPtToPage", null);
    __decorate([
        bind
    ], DocView.prototype, "lengthToPDF", null);
    __decorate([
        bind
    ], DocView.prototype, "lengthToPage", null);
    __decorate([
        bind
    ], DocView.prototype, "ptToPDF", null);
    __decorate([
        bind
    ], DocView.prototype, "pdfToPage", null);
    __decorate([
        bind
    ], DocView.prototype, "rectToPage", null);
    __decorate([
        bind
    ], DocView.prototype, "getPageByDocPt", null);
    __decorate([
        bind
    ], DocView.prototype, "autoFitWidth", null);
    __decorate([
        bind
    ], DocView.prototype, "_onOrientationChange", null);
    __decorate([
        bind
    ], DocView.prototype, "wheel", null);
    __decorate([
        bind
    ], DocView.prototype, "zoomByPinch", null);
    __decorate([
        bind
    ], DocView.prototype, "_onWheel", null);
    __decorate([
        bind
    ], DocView.prototype, "_onVerticalScroll", null);
    __decorate([
        bind
    ], DocView.prototype, "_onHorizontalScroll", null);
    __decorate([
        bind
    ], DocView.prototype, "_shouldHandleWheelX", null);
    __decorate([
        bind
    ], DocView.prototype, "_shouldHandleWheelY", null);
    __decorate([
        bind
    ], DocView.prototype, "_renderRows", null);
    __decorate([
        bind
    ], DocView.prototype, "_didScrollStart", null);
    __decorate([
        bind
    ], DocView.prototype, "_didScrollStop", null);
    __decorate([
        bind
    ], DocView.prototype, "_calcVisibleDocRect", null);
    __decorate([
        bind
    ], DocView.prototype, "_calculateState", null);
    __decorate([
        bind
    ], DocView.prototype, "_getRowPosition", null);
    __decorate([
        bind
    ], DocView.prototype, "_getRowHeight", null);
    __decorate([
        bind
    ], DocView.prototype, "_getRowWidth", null);
    __decorate([
        bind
    ], DocView.prototype, "_onMouseDown_TouchStart", null);
    __decorate([
        bind
    ], DocView.prototype, "_onMouseMoveEnd", null);
    __decorate([
        bind
    ], DocView.prototype, "_onMouseMove", null);
    return DocView;
}(Component));
export default DocView;
var HorizontalScrollbar = /** @class */ (function (_super) {
    __extends(HorizontalScrollbar, _super);
    function HorizontalScrollbar() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    HorizontalScrollbar.prototype.render = function (props) {
        return h(Scrollbar, __assign({}, props, { orientation: "horizontal", isOpaque: true }));
    };
    return HorizontalScrollbar;
}(Component));
//# sourceMappingURL=DocView.js.map