var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { h, Component } from "preact";
import * as shallowEqual from "shallow-equals";
import translateDOMPositionXY from "../vendor/translateDOMPositionXY";
import TileLayer from "./TileLayer";
import ZoomUtil from "./ZoomUtil";
import cx from "../vendor/cx";
var LayerManager = /** @class */ (function () {
    function LayerManager() {
        this.layers = [];
        this.layers = [{ zoom: 0 }, { zoom: 0 }];
        this.layers.length = 2;
    }
    LayerManager.prototype.createLayer = function (zoom) {
        if (this.layers[1].zoom != zoom) {
            this.layers[0] = this.layers[1];
            this.layers[1] = { zoom: zoom };
        }
    };
    return LayerManager;
}());
var DocViewPage = /** @class */ (function (_super) {
    __extends(DocViewPage, _super);
    function DocViewPage(props) {
        var _this = _super.call(this, props) || this;
        _this.layerManager = new LayerManager();
        _this.layerManager.createLayer(ZoomUtil.getPreferZoom(props.zoom));
        return _this;
    }
    DocViewPage.prototype.componentWillReceiveProps = function (nextProps) {
        var preferZoom = ZoomUtil.getPreferZoom(nextProps.zoom);
        if (preferZoom != this.props.zoom) {
            this.layerManager.createLayer(preferZoom);
        }
    };
    DocViewPage.prototype.shouldComponentUpdate = function (nextProps, nextState) {
        return !shallowEqual(this.state, nextState) || !shallowEqual(this.props, nextProps);
    };
    DocViewPage.prototype.getPageSizeClass = function () {
        var zoom = this.props.zoom;
        if (zoom <= 0.51) {
            return "page-xs";
        }
        if (zoom <= 0.76) {
            return "page-sm";
        }
        if (zoom <= 1.26) {
            return "page-md";
        }
        return "page-lg";
    };
    DocViewPage.prototype.render = function (props) {
        var pageStyle = {
            top: 0,
            position: "absolute",
            width: props.width,
            height: props.height,
            left: props.left,
            zIndex: 0,
        };
        translateDOMPositionXY(pageStyle, 0, props.offsetTop);
        //console.log("render page" + props.pageIndex + ":" + props.zoom)
        return (h("div", { id: "page" + props.pageIndex, class: cx("fwr-page", this.getPageSizeClass()), style: pageStyle },
            h("div", { class: "fwr-page-background" }, this.layerManager.layers.filter(function (layerInfo) { return layerInfo.zoom > 0; }).map(function (layerInfo, index) {
                return h(TileLayer, { key: "layer" + props.pageIndex + layerInfo.zoom, pageAboveRect: props.pageAboveRect, pageIndex: props.pageIndex, zoom: layerInfo.zoom, zIndex: index, showZoom: props.zoom, visiblePageRect: props.visiblePageRect });
            })),
            this.context.plugins.map(function (plugin) {
                return plugin.renderPage(props);
            })));
    };
    return DocViewPage;
}(Component));
export default DocViewPage;
//# sourceMappingURL=DocViewPage.js.map