var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { h, Component } from "preact";
require('es6-object-assign').polyfill();
var ALL_INITIALIZERS = [];
var READY_INITIALIZERS = [];
function isWebpackReady(getModuleIds) {
    if (typeof __webpack_modules__ !== 'object') {
        return false;
    }
    return getModuleIds().every(function (moduleId) {
        return (typeof moduleId !== 'undefined' &&
            typeof __webpack_modules__[moduleId] !== 'undefined');
    });
}
function load(loader) {
    var promise = loader();
    var state = {
        loading: true,
        loaded: null,
        error: null
    };
    state.promise = promise.then(function (loaded) {
        state.loading = false;
        state.loaded = loaded;
        return loaded;
    }).catch(function (err) {
        state.loading = false;
        state.error = err;
        throw err;
    });
    return state;
}
function loadMap(obj) {
    var state = {
        loading: false,
        loaded: {},
        error: null
    };
    var promises = [];
    try {
        Object.keys(obj).forEach(function (key) {
            var result = load(obj[key]);
            if (!result.loading) {
                state.loaded[key] = result.loaded;
                state.error = result.error;
            }
            else {
                state.loading = true;
            }
            promises.push(result.promise);
            result.promise.then(function (res) {
                state.loaded[key] = res;
            }).catch(function (err) {
                state.error = err;
            });
        });
    }
    catch (err) {
        state.error = err;
    }
    state.promise = Promise.all(promises).then(function (res) {
        state.loading = false;
        return res;
    }).catch(function (err) {
        state.loading = false;
        throw err;
    });
    return state;
}
function resolve(obj) {
    return obj && obj.__esModule ? obj.default : obj;
}
function render(loaded, props) {
    return h(resolve(loaded), props);
}
function createLoadableComponent(loadFn, options) {
    if (!options.loading) {
        throw new Error('react-loadable requires a `loading` component');
    }
    var opts = Object.assign({
        loader: null,
        loading: null,
        delay: 200,
        timeout: null,
        render: render,
        webpack: null,
        modules: null,
    }, options);
    var res = null;
    function init() {
        if (!res) {
            res = loadFn(opts.loader);
        }
        return res.promise;
    }
    ALL_INITIALIZERS.push(init);
    if (typeof opts.webpack === 'function') {
        READY_INITIALIZERS.push(function () {
            if (isWebpackReady(opts.webpack)) {
                return init();
            }
        });
    }
    return /** @class */ (function (_super) {
        __extends(LoadableComponent, _super);
        function LoadableComponent(props) {
            var _this = _super.call(this, props) || this;
            _this.retry = function () {
                _this.setState({ error: null, loading: true, timedOut: false });
                res = loadFn(opts.loader);
                _this._loadModule();
            };
            init();
            _this.state = {
                error: res.error,
                pastDelay: false,
                timedOut: false,
                loading: res.loading,
                loaded: res.loaded
            };
            return _this;
        }
        LoadableComponent.preload = function () {
            return init();
        };
        LoadableComponent.prototype.componentWillMount = function () {
            this._mounted = true;
            this._loadModule();
        };
        LoadableComponent.prototype._loadModule = function () {
            var _this = this;
            if (this.context.loadable && Array.isArray(opts.modules)) {
                opts.modules.forEach(function (moduleName) {
                    _this.context.loadable.report(moduleName);
                });
            }
            if (!res.loading) {
                return;
            }
            if (typeof opts.delay === 'number') {
                if (opts.delay === 0) {
                    this.setState({ pastDelay: true });
                }
                else {
                    this._delay = setTimeout(function () {
                        _this.setState({ pastDelay: true });
                    }, opts.delay);
                }
            }
            if (typeof opts.timeout === 'number') {
                this._timeout = setTimeout(function () {
                    _this.setState({ timedOut: true });
                }, opts.timeout);
            }
            var update = function () {
                if (!_this._mounted) {
                    return;
                }
                _this.setState({
                    error: res.error,
                    loaded: res.loaded,
                    loading: res.loading
                });
                _this._clearTimeouts();
            };
            res.promise.then(function () {
                update();
            }).catch(function (err) {
                update();
                throw err;
            });
        };
        LoadableComponent.prototype.componentWillUnmount = function () {
            this._mounted = false;
            this._clearTimeouts();
        };
        LoadableComponent.prototype._clearTimeouts = function () {
            clearTimeout(this._delay);
            clearTimeout(this._timeout);
        };
        LoadableComponent.prototype.render = function () {
            if (this.state.loading || this.state.error) {
                return h(opts.loading, {
                    isLoading: this.state.loading,
                    pastDelay: this.state.pastDelay,
                    timedOut: this.state.timedOut,
                    error: this.state.error,
                    retry: this.retry
                });
            }
            else if (this.state.loaded) {
                return opts.render(this.state.loaded, this.props);
            }
            else {
                return null;
            }
        };
        return LoadableComponent;
    }(Component));
}
var Loadable = function (opts) {
    return createLoadableComponent(load, opts);
};
function LoadableMap(opts) {
    if (typeof opts.render !== 'function') {
        throw new Error('LoadableMap requires a `render(loaded, props)` function');
    }
    return createLoadableComponent(loadMap, opts);
}
Loadable.Map = LoadableMap;
var Capture = /** @class */ (function (_super) {
    __extends(Capture, _super);
    function Capture() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Capture.prototype.getChildContext = function () {
        return {
            loadable: {
                report: this.props.report,
            },
        };
    };
    Capture.prototype.render = function () {
        return this.props.children[0];
    };
    return Capture;
}(Component));
Loadable.Capture = Capture;
function flushInitializers(initializers) {
    var promises = [];
    while (initializers.length) {
        var init = initializers.pop();
        promises.push(init());
    }
    return Promise.all(promises).then(function () {
        if (initializers.length) {
            return flushInitializers(initializers);
        }
    });
}
Loadable.preloadAll = function () {
    return new Promise(function (resolve, reject) {
        flushInitializers(ALL_INITIALIZERS).then(resolve, reject);
    });
};
Loadable.preloadReady = function () {
    return new Promise(function (resolve, reject) {
        // We always will resolve, errors should be handled within loading UIs.
        flushInitializers(READY_INITIALIZERS).then(resolve, resolve);
    });
};
export default Loadable;
//# sourceMappingURL=ReactLoadable.js.map