var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { h, Component, cloneElement } from "preact";
import classNames from 'classnames';
import DraggableCore from "./draggableCore";
import { canDragX, canDragY, createDraggableData, getBoundPosition } from './utils/positionFns';
import translateDOMPositionXY from "../../vendor/translateDOMPositionXY";
import bind from "../../vendor/bind";
;
;
var Draggable = /** @class */ (function (_super) {
    __extends(Draggable, _super);
    function Draggable(props) {
        var _this = _super.call(this, props) || this;
        _this.state = {
            // Whether or not we are currently dragging.
            dragging: false,
            // Whether or not we have been dragged before.
            dragged: false,
            // Current transform x and y.
            x: props.position ? props.position.x : props.defaultPosition.x,
            y: props.position ? props.position.y : props.defaultPosition.y,
            // Used for compensating for out-of-bounds drags
            slackX: 0, slackY: 0,
        };
        return _this;
    }
    Draggable.prototype.componentDidMount = function () {
    };
    Draggable.prototype.componentWillReceiveProps = function (nextProps) {
        // Set x/y if position has changed
        if (nextProps.position &&
            (!this.props.position ||
                nextProps.position.x !== this.props.position.x ||
                nextProps.position.y !== this.props.position.y)) {
            this.setState({ x: nextProps.position.x, y: nextProps.position.y });
        }
    };
    Draggable.prototype.componentWillUnmount = function () {
        this.setState({ dragging: false }); // prevents invariant if unmounted while dragging
    };
    Draggable.prototype.onDragStart = function (e, coreData) {
        // console.log('Draggable: onDragStart: %j', coreData);
        // Short-circuit if user's callback killed it.
        var shouldStart = this.props.onStart(e, createDraggableData(this, coreData));
        // Kills start event on core as well, so move handlers are never bound.
        if (shouldStart === false)
            return false;
        this.setState({ dragging: true, dragged: true });
        // e.stopPropagation();
    };
    Draggable.prototype.onDrag = function (e, coreData) {
        //  e.stopPropagation();
        if (!this.state.dragging)
            return false;
        //console.log('Draggable: onDrag: %j', coreData);    
        var uiData = createDraggableData(this, coreData);
        var newState = {
            x: uiData.x,
            y: uiData.y
        };
        // Keep within bounds.
        if (this.props.bounds) {
            // Save original x and y.
            var x = newState.x, y = newState.y;
            // Add slack to the values used to calculate bound position. This will ensure that if
            // we start removing slack, the element won't react to it right away until it's been
            // completely removed.
            newState.x += this.state.slackX;
            newState.y += this.state.slackY;
            // Get bound position. This will ceil/floor the x and y within the boundaries.
            var _a = getBoundPosition(this, newState.x, newState.y), newStateX = _a[0], newStateY = _a[1];
            newState.x = newStateX;
            newState.y = newStateY;
            // Recalculate slack by noting how much was shaved by the boundPosition handler.
            newState.slackX = this.state.slackX + (x - newState.x);
            newState.slackY = this.state.slackY + (y - newState.y);
            // Update the event we fire to reflect what really happened after bounds took effect.
            uiData.x = newState.x;
            uiData.y = newState.y;
            uiData.deltaX = newState.x - this.state.x;
            uiData.deltaY = newState.y - this.state.y;
        }
        // Short-circuit if user's callback killed it.
        var shouldUpdate = this.props.onDrag(e, uiData);
        if (shouldUpdate === false)
            return false;
        this.setState(newState);
    };
    Draggable.prototype.onDragStop = function (e, coreData) {
        //e.stopPropagation();
        if (!this.state.dragging)
            return false;
        // Short-circuit if user's callback killed it.
        var shouldStop = this.props.onStop(e, createDraggableData(this, coreData));
        if (shouldStop === false)
            return false;
        // console.log('Draggable: onDragStop: %j', coreData);
        var newState = {
            dragging: false,
            slackX: 0,
            slackY: 0
        };
        // If this is a controlled component, the result of this operation will be to
        // revert back to the old position. We expect a handler on `onDragStop`, at the least.
        var controlled = Boolean(this.props.position);
        if (controlled) {
            var _a = this.props.position, x = _a.x, y = _a.y;
            newState.x = x;
            newState.y = y;
        }
        this.setState(newState);
    };
    Draggable.prototype.render = function () {
        var style = {};
        var controlled = Boolean(this.props.position);
        var draggable = !controlled || this.state.dragging;
        var position = this.props.position || this.props.defaultPosition;
        var transformOpts = {
            // Set left if horizontal drag is enabled
            x: canDragX(this.props) && draggable ?
                this.state.x :
                position.x,
            // Set top if vertical drag is enabled
            y: canDragY(this.props) && draggable ?
                this.state.y :
                position.y
        };
        var _a = this.props, defaultClassName = _a.defaultClassName, defaultClassNameDragging = _a.defaultClassNameDragging, defaultClassNameDragged = _a.defaultClassNameDragged;
        var children = this.props.children[0];
        var className = classNames((children.attributes.class || ''), defaultClassName, (_b = {},
            _b[defaultClassNameDragging] = this.state.dragging,
            _b[defaultClassNameDragged] = this.state.dragged,
            _b));
        translateDOMPositionXY(style, transformOpts.x, transformOpts.y);
        return (h(DraggableCore, __assign({}, this.props, { onStart: this.onDragStart, onDrag: this.onDrag, onStop: this.onDragStop }), cloneElement(children, {
            class: className,
            style: __assign({}, children.attributes.style, style),
        })));
        var _b;
    };
    Draggable.defaultProps = __assign({}, DraggableCore.defaultProps, { axis: 'both', bounds: false, defaultClassName: 'react-draggable', defaultClassNameDragging: 'react-draggable-dragging', defaultClassNameDragged: 'react-draggable-dragged', defaultPosition: { x: 0, y: 0 }, position: null });
    __decorate([
        bind
    ], Draggable.prototype, "onDragStart", null);
    __decorate([
        bind
    ], Draggable.prototype, "onDrag", null);
    __decorate([
        bind
    ], Draggable.prototype, "onDragStop", null);
    return Draggable;
}(Component));
export default Draggable;
//# sourceMappingURL=draggable.js.map