var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { Component, cloneElement } from "preact";
import bind from "../../vendor/bind";
import { addUserSelectStyles, removeUserSelectStyles, getTouchIdentifier, styleHacks } from "./utils/domFns";
import { createCoreData, getControlPosition } from './utils/positionFns';
import EventListener from '../../vendor/EventListener';
var eventsFor = {
    touch: {
        start: 'touchstart',
        move: 'touchmove',
        stop: 'touchend'
    },
    mouse: {
        start: 'mousedown',
        move: 'mousemove',
        stop: 'mouseup'
    }
};
// Default to mouse events.
var dragEventFor = eventsFor.mouse;
var DraggableCore = /** @class */ (function (_super) {
    __extends(DraggableCore, _super);
    function DraggableCore(props) {
        var _this = _super.call(this, props) || this;
        _this.state = {
            dragging: false,
            // Used while dragging to determine deltas.
            lastX: NaN, lastY: NaN,
            touchIdentifier: null
        };
        return _this;
    }
    DraggableCore.prototype.componentDidMount = function () {
    };
    DraggableCore.prototype.componentWillUnmount = function () {
        // Remove any leftover event handlers. Remove both touch and mouse handlers in case
        // some browser quirk caused a touch event to fire during a mouse move, or vice versa. 
        var thisNode = this.base;
        if (thisNode) {
            var ownerDocument = thisNode.ownerDocument;
            this.removeEvent();
            if (this.props.enableUserSelectHack)
                removeUserSelectStyles(ownerDocument);
        }
    };
    DraggableCore.prototype.handleDragStart = function (e) {
        // console.log("dragstart")
        // Make it possible to attach event handlers on top of this one.
        this.props.onMouseDown(e);
        // Only accept left-clicks.
        if (!this.props.allowAnyClick && typeof e.button === 'number' && e.button !== 0)
            return false;
        // Get nodes. Be sure to grab relative document (could be iframed)
        var thisNode = this.base;
        if (!thisNode || !thisNode.ownerDocument || !thisNode.ownerDocument.body) {
            throw new Error('<DraggableCore> not mounted on DragStart!');
        }
        var ownerDocument = thisNode.ownerDocument;
        // Set touch identifier in component state if this is a touch event. This allows us to
        // distinguish between individual touches on multitouch screens by identifying which
        // touchpoint was set to this element.
        var touchIdentifier = getTouchIdentifier(e);
        this.setState({ touchIdentifier: touchIdentifier });
        // Get the current drag point from the event. This is used as the offset.
        var position = getControlPosition(e, touchIdentifier, this);
        if (position == null)
            return; // not possible but satisfies flow
        var x = position.x, y = position.y;
        // Create an event object with all the data parents need to make a decision here.
        var coreEvent = createCoreData(this, x, y);
        //  console.log('DraggableCore: handleDragStart: %j', coreEvent);
        // Call event handler. If it returns explicit false, cancel.
        //console.log('calling', this.props.onStart);
        var shouldUpdate = this.props.onStart(e, coreEvent);
        if (shouldUpdate === false)
            return;
        // Add a style to the body to disable user-select. This prevents text from
        // being selected all over the page.
        if (this.props.enableUserSelectHack)
            addUserSelectStyles(ownerDocument);
        // Initiate dragging. Set the current x and y as offsets
        // so we know how much we've moved during the drag. This allows us
        // to drag elements around even if they have been moved, without issue.
        this.setState({
            dragging: true,
            lastX: x,
            lastY: y
        });
        // Add events to the document directly so we catch when the user's mouse/touch moves outside of
        // this element. We use different events depending on whether or not we have detected that this
        // is a touch-capable device.  
        this.addEvent(ownerDocument);
    };
    ;
    DraggableCore.prototype.addEvent = function (element) {
        this._eventMoveToken = EventListener.listen(element, dragEventFor.move, this.handleDrag);
        this._eventStopToken = EventListener.listen(element, dragEventFor.stop, this.handleDragStop);
    };
    DraggableCore.prototype.removeEvent = function () {
        if (this._eventMoveToken) {
            this._eventMoveToken.remove();
            this._eventMoveToken = null;
        }
        if (this._eventStopToken) {
            this._eventStopToken.remove();
            this._eventStopToken = null;
        }
    };
    DraggableCore.prototype.handleDrag = function (e) {
        // Prevent scrolling on mobile devices, like ipad/iphone.
        if (e.type == 'touchmove')
            e.preventDefault();
        // Get the current drag point from the event. This is used as the offset.
        var position = getControlPosition(e, this.state.touchIdentifier, this);
        if (position == null)
            return;
        var x = position.x, y = position.y;
        var coreEvent = createCoreData(this, x, y);
        // console.log('DraggableCore: handleDrag: %j', coreEvent);
        // Call event handler. If it returns explicit false, trigger end.
        var shouldUpdate = this.props.onDrag(e, coreEvent);
        if (shouldUpdate === false) {
            try {
                this.handleDragStop(new MouseEvent('mouseup'));
            }
            catch (err) {
                // Old browsers
                var event_1 = (document.createEvent('MouseEvents'));
                // I see why this insanity was deprecated
                // $FlowIgnore
                event_1.initMouseEvent('mouseup', true, true, window, 0, 0, 0, 0, 0, false, false, false, false, 0, null);
                this.handleDragStop(event_1);
            }
            return;
        }
        this.setState({
            lastX: x,
            lastY: y
        });
    };
    ;
    DraggableCore.prototype.handleDragStop = function (e) {
        if (!this.state.dragging)
            return;
        var position = getControlPosition(e, this.state.touchIdentifier, this);
        if (position == null)
            return;
        var x = position.x, y = position.y;
        var coreEvent = createCoreData(this, x, y);
        var thisNode = this.base;
        if (thisNode) {
            // Remove user-select hack
            if (this.props.enableUserSelectHack)
                removeUserSelectStyles(thisNode.ownerDocument);
        }
        //console.log('DraggableCore: handleDragStop: %j', coreEvent);
        // Reset the el.
        this.setState({
            dragging: false,
            lastX: NaN,
            lastY: NaN
        });
        // Call event handler
        this.props.onStop(e, coreEvent);
        if (thisNode) {
            // Remove event handlers
            //console.log('DraggableCore: Removing handlers');
            this.removeEvent();
        }
    };
    ;
    DraggableCore.prototype.onMouseDown = function (e) {
        dragEventFor = eventsFor.mouse; // on touchscreen laptops we could switch back to mouse
        return this.handleDragStart(e);
    };
    ;
    DraggableCore.prototype.onMouseUp = function (e) {
        dragEventFor = eventsFor.mouse;
        return this.handleDragStop(e);
    };
    ;
    // Same as onMouseDown (start drag), but now consider this a touch device.
    DraggableCore.prototype.onTouchStart = function (e) {
        // We're on a touch device now, so change the event handlers
        e.preventDefault();
        dragEventFor = eventsFor.touch;
        return this.handleDragStart(e);
    };
    ;
    DraggableCore.prototype.onTouchEnd = function (e) {
        // We're on a touch device now, so change the event handlers
        dragEventFor = eventsFor.touch;
        return this.handleDragStop(e);
    };
    ;
    DraggableCore.prototype.render = function () {
        var _this = this;
        var children = this.props.children;
        return children.map(function (element) {
            return cloneElement(element, {
                style: styleHacks(element.attributes.style),
                onMouseDown: _this.onMouseDown,
                onTouchStart: _this.onTouchStart,
                onMouseUp: _this.onMouseUp,
                onTouchEnd: _this.onTouchEnd
            });
        })[0];
    };
    DraggableCore.defaultProps = {
        allowAnyClick: false,
        cancel: null,
        disabled: false,
        enableUserSelectHack: true,
        offsetParent: null,
        handle: null,
        transform: null,
        onStart: function () { },
        onDrag: function () { },
        onStop: function () { },
        onMouseDown: function () { }
    };
    __decorate([
        bind
    ], DraggableCore.prototype, "handleDrag", null);
    __decorate([
        bind
    ], DraggableCore.prototype, "handleDragStop", null);
    __decorate([
        bind
    ], DraggableCore.prototype, "onMouseDown", null);
    __decorate([
        bind
    ], DraggableCore.prototype, "onMouseUp", null);
    __decorate([
        bind
    ], DraggableCore.prototype, "onTouchStart", null);
    __decorate([
        bind
    ], DraggableCore.prototype, "onTouchEnd", null);
    return DraggableCore;
}(Component));
export default DraggableCore;
//# sourceMappingURL=draggableCore.js.map