var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
import { findInArray, isFunction, int } from './shims';
import browserPrefix, { browserPrefixToKey } from './getPrefix';
var matchesSelectorFunc = '';
export function matchesSelector(el, selector) {
    if (!matchesSelectorFunc) {
        matchesSelectorFunc = findInArray([
            'matches',
            'webkitMatchesSelector',
            'mozMatchesSelector',
            'msMatchesSelector',
            'oMatchesSelector'
        ], function (method) {
            // $FlowIgnore: Doesn't think elements are indexable
            return isFunction(el[method]);
        });
    }
    // Might not be found entirely (not an Element?) - in that case, bail
    // $FlowIgnore: Doesn't think elements are indexable
    if (!isFunction(el[matchesSelectorFunc]))
        return false;
    // $FlowIgnore: Doesn't think elements are indexable
    return el[matchesSelectorFunc](selector);
}
// Works up the tree to the draggable itself attempting to match selector.
export function matchesSelectorAndParentsTo(el, selector, baseNode) {
    var node = el;
    do {
        if (matchesSelector(node, selector))
            return true;
        if (node === baseNode)
            return false;
        node = node.parentNode;
    } while (node);
    return false;
}
export function outerHeight(node) {
    // This is deliberately excluding margin for our calculations, since we are using
    // offsetTop which is including margin. See getBoundPosition
    var height = node.clientHeight;
    var computedStyle = node.ownerDocument.defaultView.getComputedStyle(node);
    height += int(computedStyle.borderTopWidth);
    height += int(computedStyle.borderBottomWidth);
    return height;
}
export function outerWidth(node) {
    // This is deliberately excluding margin for our calculations, since we are using
    // offsetLeft which is including margin. See getBoundPosition
    var width = node.clientWidth;
    var computedStyle = node.ownerDocument.defaultView.getComputedStyle(node);
    width += int(computedStyle.borderLeftWidth);
    width += int(computedStyle.borderRightWidth);
    return width;
}
export function innerHeight(node) {
    var height = node.clientHeight;
    var computedStyle = node.ownerDocument.defaultView.getComputedStyle(node);
    height -= int(computedStyle.paddingTop);
    height -= int(computedStyle.paddingBottom);
    return height;
}
export function innerWidth(node) {
    var width = node.clientWidth;
    var computedStyle = node.ownerDocument.defaultView.getComputedStyle(node);
    width -= int(computedStyle.paddingLeft);
    width -= int(computedStyle.paddingRight);
    return width;
}
// Get from offsetParent
export function offsetXYFromParent(evt, offsetParent) {
    var isBody = offsetParent === offsetParent.ownerDocument.body;
    var offsetParentRect = isBody ? { left: 0, top: 0 } : offsetParent.getBoundingClientRect();
    var x = evt.clientX + offsetParent.scrollLeft - offsetParentRect.left;
    var y = evt.clientY + offsetParent.scrollTop - offsetParentRect.top;
    return { x: x, y: y };
}
export function createCSSTransform(_a) {
    var x = _a.x, y = _a.y;
    // Replace unitless items with px
    return _b = {}, _b[browserPrefixToKey('transform', browserPrefix)] = 'translate(' + x + 'px,' + y + 'px)', _b;
    var _b;
}
export function createSVGTransform(_a) {
    var x = _a.x, y = _a.y;
    return 'translate(' + x + ',' + y + ')';
}
export function getTouch(e, identifier) {
    return (e.targetTouches && findInArray(e.targetTouches, function (t) { return identifier === t.identifier; })) ||
        (e.changedTouches && findInArray(e.changedTouches, function (t) { return identifier === t.identifier; }));
}
export function getTouchIdentifier(e) {
    if (e.targetTouches && e.targetTouches[0])
        return e.targetTouches[0].identifier;
    if (e.changedTouches && e.changedTouches[0])
        return e.changedTouches[0].identifier;
}
// User-select Hacks:
//
// Useful for preventing blue highlights all over everything when dragging.
// Note we're passing `document` b/c we could be iframed
export function addUserSelectStyles(doc) {
    if (!doc)
        return;
    var styleEl = doc.getElementById('react-draggable-style-el');
    if (!styleEl) {
        styleEl = doc.createElement('style');
        styleEl.type = 'text/css';
        styleEl.id = 'react-draggable-style-el';
        styleEl.innerHTML = '.react-draggable-transparent-selection *::-moz-selection {background: transparent;}\n';
        styleEl.innerHTML += '.react-draggable-transparent-selection *::selection {background: transparent;}\n';
        doc.getElementsByTagName('head')[0].appendChild(styleEl);
    }
    if (doc.body)
        addClassName(doc.body, 'react-draggable-transparent-selection');
}
export function removeUserSelectStyles(doc) {
    try {
        if (doc && doc.body)
            removeClassName(doc.body, 'react-draggable-transparent-selection');
        // $FlowIgnore: IE
        if (doc.selection) {
            // $FlowIgnore: IE
            doc.selection.empty();
        }
        else {
            window.getSelection().removeAllRanges(); // remove selection caused by scroll
        }
    }
    catch (e) {
        // probably IE
    }
}
export function styleHacks(childStyle) {
    if (childStyle === void 0) { childStyle = {}; }
    // Workaround IE pointer events; see #51
    // https://github.com/mzabriskie/react-draggable/issues/51#issuecomment-103488278
    return __assign({ touchAction: 'none' }, childStyle);
}
export function addClassName(el, className) {
    if (el.classList) {
        el.classList.add(className);
    }
    else {
        if (!el.className.match(new RegExp("(?:^|\\s)" + className + "(?!\\S)"))) {
            el.className += " " + className;
        }
    }
}
export function removeClassName(el, className) {
    if (el.classList) {
        el.classList.remove(className);
    }
    else {
        el.className = el.className.replace(new RegExp("(?:^|\\s)" + className + "(?!\\S)", 'g'), '');
    }
}
//# sourceMappingURL=domFns.js.map