var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { h, Component } from "preact";
import Keys from "../vendor/Key";
import cssVar from "../vendor/cssVar";
import DOMMouseMoveTracker from "../vendor/DOMMouseMoveTracker";
import translateDOMPositionXY from "../vendor/translateDOMPositionXY";
import WheelHandler from "./WheelHandler";
import cx from "../vendor/cx";
import { getWheelEventType } from "../vendor/normalizeWheel";
import EventListener from '../vendor/EventListener';
function emptyFunction() { }
var defaults = {
    "isOpaque": false,
    "orientation": "vertical",
};
var UNSCROLLABLE_STATE = {
    position: 0,
    scrollable: false,
};
var FACE_MARGIN = parseInt(cssVar('scrollbar-face-margin'), 10);
var FACE_MARGIN_2 = FACE_MARGIN * 2;
var FACE_SIZE_MIN = 30;
var KEYBOARD_SCROLL_AMOUNT = 40;
var _lastScrolledScrollbar = null;
var Scrollbar = /** @class */ (function (_super) {
    __extends(Scrollbar, _super);
    function Scrollbar(props) {
        var _this = _super.call(this, props) || this;
        _this.namespace = ".scrollbar";
        _this.scrollTo = null;
        _this.scrollx = {};
        _this.scrolly = {};
        _this._updating = false;
        _this._onMouseDown = _this._onMouseDown.bind(_this);
        _this._onMouseMove = _this._onMouseMove.bind(_this);
        _this._onMouseMoveEnd = _this._onMouseMoveEnd.bind(_this);
        _this._onFocus = _this._onFocus.bind(_this);
        _this._onBlur = _this._onBlur.bind(_this);
        _this._shouldHandleX = _this._shouldHandleX.bind(_this);
        _this._shouldHandleY = _this._shouldHandleY.bind(_this);
        _this._shouldHandleChange = _this._shouldHandleChange.bind(_this);
        _this._onWheel = _this._onWheel.bind(_this);
        _this._onWheelX = _this._onWheelX.bind(_this);
        _this._onWheelY = _this._onWheelY.bind(_this);
        _this._onKeyDown = _this._onKeyDown.bind(_this);
        _this._visible = true;
        _this.state = _this._calculateState(props.position || props.defaultPosition || 0, props.size, props.contentSize, props.orientation);
        return _this;
    }
    Scrollbar.prototype.componentWillMount = function () {
        var isHorizontal = this.props.orientation === 'horizontal';
        var onWheel = isHorizontal ? this._onWheelX : this._onWheelY;
        this._wheelHandler = new WheelHandler(onWheel, this._shouldHandleX, // Should hanlde horizontal scroll
        this._shouldHandleY // Should handle vertical scroll
        );
    };
    Scrollbar.prototype.componentDidMount = function () {
        this._isMounted = true;
        this._mouseMoveTracker = new DOMMouseMoveTracker(this._onMouseMove, this._onMouseMoveEnd, document.documentElement);
        this._eventDownToken = EventListener.listen(this.base, 'mousedown', this._onMouseDown);
        this._eventWheelToken = EventListener.listen(this.base, getWheelEventType(), this._wheelHandler.onWheel);
    };
    Scrollbar.prototype.componentWillUnmount = function () {
        this._isMounted = false;
        if (this._eventDownToken) {
            this._eventDownToken.remove();
            this._eventDownToken = null;
        }
        if (this._eventWheelToken) {
            this._eventWheelToken.remove();
            this._eventWheelToken = null;
        }
    };
    Scrollbar.prototype._onClick = function (e) {
        e.stopPropagation();
    };
    Scrollbar.prototype._calculateState = function (position, size, contentSize, orientation) {
        if (size < 1 || contentSize <= size) {
            return UNSCROLLABLE_STATE;
        }
        var stateKey = position + "_" + size + "_" + contentSize + "_" + orientation;
        if (this._stateKey == stateKey) {
            return this._stateForKey;
        }
        // There are two types of positions here.
        // 1) Phisical position: changed by mouse / keyboard
        // 2) Logical position: changed by props.
        // The logical position will be kept as as internal state and the `render()`
        // function will translate it into physical position to render.
        var isHorizontal = orientation === 'horizontal';
        var scale = size / contentSize;
        var faceSize = size * scale;
        if (faceSize < FACE_SIZE_MIN) {
            scale = (size - FACE_SIZE_MIN) / (contentSize - size);
            faceSize = FACE_SIZE_MIN;
        }
        var scrollable = true;
        var maxPosition = contentSize - size;
        if (position < 0) {
            position = 0;
        }
        else if (position > maxPosition) {
            position = maxPosition;
        }
        var isDragging = this._mouseMoveTracker ?
            this._mouseMoveTracker.isDragging() :
            false;
        // This function should only return flat values that can be compared quiclky
        // by `ReactComponentWithPureRenderMixin`.
        var state = {
            faceSize: faceSize,
            isDragging: isDragging,
            isHorizontal: isHorizontal,
            position: position,
            scale: scale,
            scrollable: scrollable,
            contentSize: contentSize,
            size: size
        };
        // cache the state for later use.
        this._stateKey = stateKey;
        this._stateForKey = state;
        return state;
    };
    Scrollbar.prototype._onWheel = function (delta) {
        var props = this.props;
        // The mouse may move faster then the animation frame does.
        // Use `requestAnimationFrame` to avoid over-updating.
        this._setNextState(this._calculateState(this.state.position + delta, props.size, props.contentSize, props.orientation));
    };
    Scrollbar.prototype._onMouseMoveEnd = function () {
        this._nextState = null;
        this._mouseMoveTracker.releaseMouseMoves();
        this.setState({ isDragging: false });
    };
    Scrollbar.prototype._onMouseMove = function (deltaX, deltaY) {
        var props = this.props;
        var delta = this.state.isHorizontal ? deltaX : deltaY;
        delta /= this.state.scale;
        this._setNextState(this._calculateState(this.state.position + delta, props.size, props.contentSize, props.orientation));
    };
    Scrollbar.prototype._shouldHandleChange = function (delta) {
        var nextState = this._calculateState(this.state.position + delta, this.props.size, this.props.contentSize, this.props.orientation);
        return nextState.position !== this.state.position;
    };
    Scrollbar.prototype._shouldHandleX = function (delta) {
        return this.props.orientation === 'horizontal' ?
            this._shouldHandleChange(delta) :
            false;
    };
    Scrollbar.prototype._shouldHandleY = function (delta) {
        return this.props.orientation !== 'horizontal' ?
            this._shouldHandleChange(delta) :
            false;
    };
    Scrollbar.prototype._onWheelY = function (deltaX, deltaY) {
        this._onWheel(deltaY);
    };
    Scrollbar.prototype._onWheelX = function (deltaX, deltaY) {
        this._onWheel(deltaX);
    };
    Scrollbar.prototype._onMouseDown = function (event) {
        var nextState;
        if (event.target !== this.face) {
            // Both `offsetX` and `layerX` are non-standard DOM property but they are
            // magically available for browsers somehow.
            var nativeEvent = event;
            var position = this.state.isHorizontal ?
                nativeEvent.offsetX || nativeEvent.layerX :
                nativeEvent.offsetY || nativeEvent.layerY;
            // MouseDown on the scroll-track directly, move the center of the
            // scroll-face to the mouse position.
            var state = this.state;
            position /= state.scale;
            nextState = this._calculateState(position - (state.faceSize * 0.5 / state.scale), this.props.size, this.props.contentSize, this.props.orientation);
        }
        else {
            nextState = {};
        }
        nextState.focused = true;
        this._setNextState(nextState);
        this._mouseMoveTracker.captureMouseMoves(event); // Focus the node so it may receive keyboard event.
        this.base.focus();
        event.stopPropagation();
    };
    Scrollbar.prototype._onKeyDown = function (event) {
        var keyCode = event.keyCode;
        if (keyCode === Keys.TAB) {
            // Let focus move off the scrollbar.
            return;
        }
        var distance = KEYBOARD_SCROLL_AMOUNT;
        var direction = 0;
        if (this.state.isHorizontal) {
            switch (keyCode) {
                case Keys.HOME:
                    direction = -1;
                    distance = this.props.contentSize;
                    break;
                case Keys.LEFT:
                    direction = -1;
                    break;
                case Keys.RIGHT:
                    direction = 1;
                    break;
                default:
                    return;
            }
        }
        if (!this.state.isHorizontal) {
            switch (keyCode) {
                case Keys.SPACE:
                    if (event.shiftKey) {
                        direction = -1;
                    }
                    else {
                        direction = 1;
                    }
                    break;
                case Keys.HOME:
                    direction = -1;
                    distance = this.props.contentSize;
                    break;
                case Keys.UP:
                    direction = -1;
                    break;
                case Keys.DOWN:
                    direction = 1;
                    break;
                case Keys.PAGE_UP:
                    direction = -1;
                    distance = this.props.size;
                    break;
                case Keys.PAGE_DOWN:
                    direction = 1;
                    distance = this.props.size;
                    break;
                default:
                    return;
            }
        }
        event.preventDefault();
        var props = this.props;
        this._setNextState(this._calculateState(this.state.position + (distance * direction), props.size, props.contentSize, props.orientation));
    };
    Scrollbar.prototype._onFocus = function () {
        this.setState({ focused: true });
    };
    Scrollbar.prototype._onBlur = function () {
        this.setState({ focused: false });
    };
    Scrollbar.prototype._blur = function () {
        try {
            this._onBlur();
            this.base.blur();
        }
        catch (oops) {
            // pass
        }
    };
    Scrollbar.prototype._setNextState = function (nextState, props) {
        props = props || this.props;
        var controlledPosition = props.position;
        var willScroll = this.state.position !== nextState.position;
        if (controlledPosition === undefined) {
            var callback = willScroll ? this._didScroll : undefined;
            this.setState(nextState, callback);
        }
        else if (controlledPosition === nextState.position) {
            this.setState(nextState);
        }
        else {
            // Scrolling is controlled. Don't update the state and let the owner
            // to update the scrollbar instead.
            if (nextState.position !== undefined &&
                nextState.position !== this.state.position) {
                this.props.onScroll(nextState.position);
            }
            return;
        }
        if (willScroll && _lastScrolledScrollbar !== this) {
            _lastScrolledScrollbar && _lastScrolledScrollbar._blur();
            _lastScrolledScrollbar = this;
        }
    };
    Scrollbar.prototype._didScroll = function () {
        this.props.onScroll(this.state.position);
    };
    Scrollbar.prototype.componentWillReceiveProps = function (nextProps) {
        var controlledPosition = nextProps.position;
        if (controlledPosition === undefined) {
            this._setNextState(this._calculateState(this.state.position, nextProps.size, nextProps.contentSize, nextProps.orientation));
        }
        else {
            this._setNextState(this._calculateState(controlledPosition, nextProps.size, nextProps.contentSize, nextProps.orientation), nextProps);
        }
    };
    Scrollbar.prototype.render = function (props, state) {
        var _this = this;
        if (!this.state.scrollable) {
            return null;
        }
        var size = props.size;
        var isHorizontal = this.state.isHorizontal;
        var isVertical = !isHorizontal;
        var faceSize = state.faceSize;
        var isOpaque = props.isOpaque;
        var verticalTop = this.props.verticalTop || 0;
        var isActive = state.focused || state.isDragging;
        var position = state.position * state.scale + FACE_MARGIN;
        var mainClassName = {
            "ScrollbarLayout_main": true,
            "ScrollbarLayout_mainVertical": isVertical,
            "ScrollbarLayout_mainHorizontal": isHorizontal,
            "public_Scrollbar_main": true,
            "public_Scrollbar_mainOpaque": isOpaque,
            "public_Scrollbar_mainActive": isActive,
        };
        var faceClassName = {
            "ScrollbarLayout_face": true,
            'ScrollbarLayout/faceHorizontal': isHorizontal,
            'ScrollbarLayout/faceVertical': isVertical,
            "public_Scrollbar_face": true,
            'public_Scrollbar_faceActive': isActive
        };
        var mainStyle;
        var faceStyle;
        if (isHorizontal) {
            mainStyle = {
                width: size,
            };
            faceStyle = {
                width: faceSize - FACE_MARGIN_2
            };
            translateDOMPositionXY(faceStyle, position, 0);
        }
        else {
            mainStyle = {
                top: verticalTop,
                height: size,
            };
            faceStyle = {
                height: faceSize - FACE_MARGIN_2,
            };
            translateDOMPositionXY(faceStyle, 0, position);
        }
        mainStyle.zIndex = props.zIndex;
        return (h("div", { tabIndex: 0, style: mainStyle, class: cx(mainClassName) },
            h("div", { ref: function (ref) { return _this.face = ref; }, style: faceStyle, class: cx(faceClassName) })));
    };
    Scrollbar.SIZE = parseInt(cssVar('scrollbar-size'), 10);
    Scrollbar.defaultProps = {
        defaultPosition: 0,
        isOpaque: false,
        onScroll: emptyFunction,
        orientation: 'vertical',
        zIndex: 99,
    };
    return Scrollbar;
}(Component));
export default Scrollbar;
//# sourceMappingURL=Scrollbar.js.map