var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { h, Component } from "preact";
import Tile, { TileInfo } from "./Tile";
import Rect from "./Rect";
import ZoomUtil from "./ZoomUtil";
// import Watermask from "./watermask";
import { loadTileImage, TILE_SIZE } from "./loadTileImage";
import Loading from "./Loading";
import * as querySelectorAll from "dom-helpers/query/querySelectorAll";
import bind from "../vendor/bind";
import { debounce } from "throttle-debounce";
import cx from "../vendor/cx";
var TileLayer = /** @class */ (function (_super) {
    __extends(TileLayer, _super);
    function TileLayer(props, context) {
        var _this = _super.call(this, props) || this;
        _this._tiles = [];
        var pageIndex = props.pageIndex, zoom = props.zoom;
        var pageSize = context.sizeHelper.getPageSize(pageIndex, zoom);
        _this.createTiles({ pageIndex: pageIndex, pageWidth: pageSize.width, pageHeight: pageSize.height - 20 });
        _this.onTileLoading = debounce(50, _this.onTileLoading.bind(_this));
        _this.state = {
            parsed: false,
            loaded: true,
        };
        return _this;
    }
    TileLayer.prototype.componentDidMount = function () {
        var _this = this;
        this._isMounted = true;
        var _a = this.props, pageIndex = _a.pageIndex, zoom = _a.zoom;
        var _b = this.context, userInfo = _b.userInfo, docId = _b.docId, serverUrl = _b.serverUrl, manifestInfo = _b.manifestInfo, autoParse = _b.autoParse, imageWithForm = _b.imageWithForm, imageWithAnnot = _b.imageWithAnnot;
        this._canelToken = loadTileImage({
            pageIndex: pageIndex,
            rowIndex: 0,
            colIndex: 0,
            zoomName: ZoomUtil.getName(zoom),
            tileSize: TILE_SIZE,
            serverUrl: serverUrl,
            token: userInfo.token,
            docId: docId,
            manifestInfo: manifestInfo,
            autoParse: autoParse,
            imageWithAnnot: imageWithAnnot,
            imageWithForm: imageWithForm
        }, function (url) {
            _this._isMounted && _this.setState({ parsed: true });
        });
    };
    TileLayer.prototype.componentWillUnmount = function () {
        this._isMounted = false;
        this._canelToken && this._canelToken();
        this._checkLoadingToken && clearInterval(this._checkLoadingToken);
    };
    TileLayer.prototype.finishedLoaded = function () {
        if (!this._isMounted) {
            return true;
        }
        if (querySelectorAll(this.base, "img.image_unload").length == 0) {
            return true;
        }
        return false;
    };
    TileLayer.prototype.onTileLoaded = function () {
        if (this.finishedLoaded()) {
            this.setState({ loaded: true });
            if (this._checkLoadingToken) {
                clearInterval(this._checkLoadingToken);
                this._checkLoadingToken = null;
            }
        }
    };
    TileLayer.prototype.onTileLoading = function () {
        if (!this.finishedLoaded()) {
            this.setState({ loaded: false });
            if (!this._checkLoadingToken) {
                this._checkLoadingToken = setInterval(this.onTileLoaded, 10);
            }
        }
    };
    TileLayer.prototype.createTiles = function (_a) {
        var pageIndex = _a.pageIndex, pageWidth = _a.pageWidth, pageHeight = _a.pageHeight;
        var rowCount = pageHeight % TILE_SIZE == 0 ? pageHeight / TILE_SIZE : Math.floor(pageHeight / TILE_SIZE) + 1;
        var colCount = pageWidth % TILE_SIZE == 0 ? pageWidth / TILE_SIZE : Math.floor(pageWidth / TILE_SIZE) + 1;
        for (var r = 0; r < rowCount; r++) {
            for (var c = 0; c < colCount; c++) {
                var pos = new Rect({
                    left: c * TILE_SIZE,
                    top: r * TILE_SIZE,
                    width: Math.min(pageWidth - c * TILE_SIZE, TILE_SIZE),
                    height: Math.min(pageHeight - r * TILE_SIZE, TILE_SIZE)
                });
                this._tiles.push(new TileInfo({ row: r, col: c, pos: pos }));
            }
        }
    };
    TileLayer.prototype.getTileToRender = function (visiblePageRect, scale, zoom, pageIndex) {
        var _this = this;
        // console.log("getTileToRender", pageIndex, scale, zoom )
        return this._tiles
            .filter(function (tileInfo) {
            return tileInfo.position(scale).collides(visiblePageRect);
        })
            .map(function (tileInfo) {
            return h(Tile, { onLoading: _this.onTileLoading, key: "tile" + pageIndex + (zoom + "_" + tileInfo.row + "_" + tileInfo.col), pageIndex: pageIndex, zoomName: ZoomUtil.getName(zoom), col: tileInfo.col, row: tileInfo.row, tileSize: TILE_SIZE, position: tileInfo.position(scale) });
        });
    };
    // static addWaterMask({ rowIndex, colIndex, position, width, height, watermark }, $image, $container) {
    //   if (!Watermask.supportCanvas) return;
    //   var img = $image[0];
    //   setTimeout(() => {
    //     try {
    //       var $canvas = $(`<canvas class="tiles-${rowIndex}-${colIndex}"></canvas>`).css({
    //         position: "absolute",
    //         top: position.top,
    //         left: position.left,
    //         width: position.width,
    //         height: position.height,
    //         display: "none"
    //       }).appendTo($container);
    //       var canvas: any = $canvas[0];
    //       canvas.width = width;
    //       canvas.height = height;
    //       var ctx = canvas.getContext("2d");
    //       ctx.drawImage(img, 0, 0);
    //       ctx.font = "normal normal bold 18px Arial";
    //       ctx.fillStyle = "rgba(0,0,0,0.5)";
    //       ctx.textAlign = "center";
    //       ctx.save()
    //       ctx.translate(200, 200);
    //       ctx.rotate(345 * Math.PI / 180);
    //       ctx.fillText(watermark, 0, 0);
    //       ctx.restore()
    //       $image.remove();
    //       $canvas.show();
    //     }
    //     catch (error) {
    //       Watermask.supportCanvas = false;
    //     }
    //   }, 0);
    // }
    TileLayer.prototype.render = function (props, state) {
        var visiblePageRect = props.visiblePageRect, pageIndex = props.pageIndex, zoom = props.zoom, pageAboveRect = props.pageAboveRect, showZoom = props.showZoom, zIndex = props.zIndex;
        var parsed = state.parsed, loaded = state.loaded;
        //console.log("render Layer", pageIndex, zoom, showZoom, parsed)
        return (h("div", { class: cx("tiles-layer tiles-layer-" + ZoomUtil.getName(zoom), { "unload": !loaded }), style: { zIndex: zIndex } },
            parsed && this.getTileToRender(visiblePageRect, showZoom / zoom, zoom, pageIndex),
            (!loaded || !parsed) && (zIndex == 0) && h(Loading, { height: pageAboveRect.height, top: pageAboveRect.top })));
    };
    __decorate([
        bind
    ], TileLayer.prototype, "finishedLoaded", null);
    __decorate([
        bind
    ], TileLayer.prototype, "onTileLoaded", null);
    return TileLayer;
}(Component));
export default TileLayer;
//# sourceMappingURL=TileLayer.js.map