var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { h } from "preact";
import { WidgetFlags, Alignment, FieldFlags } from "../../form/models";
import WidgetBase from "./WidgetBase";
import translateDOMPositionXY from "../../vendor/translateDOMPositionXY";
import bind from "../../vendor/bind";
import { getPixelsPerPoint, parseColor } from "../../utils/common";
var WidgetText = /** @class */ (function (_super) {
    __extends(WidgetText, _super);
    function WidgetText() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    WidgetText.prototype.onFocus = function () {
        this.context.updateWidget(this.props.data, { focus: true });
    };
    WidgetText.prototype.onBlur = function () {
        this.context.updateWidget(this.props.data, { focus: false });
    };
    WidgetText.prototype.onChange = function (event) {
        this.context.setItemValue(this.props.data.name, event.target.value);
    };
    WidgetText.prototype.getAlignment = function (alignment) {
        if (alignment == Alignment.AlignmentRight) {
            return "right";
        }
        if (alignment == Alignment.AlignmentCenter) {
            return "center";
        }
        return "left";
    };
    WidgetText.prototype.formateViewValue = function (srcValue) {
        var viewValue = srcValue;
        var formatters = this.props.data.formatters;
        formatters.forEach(function (formateFunc) {
            viewValue = formateFunc(viewValue);
        });
        return viewValue;
    };
    WidgetText.prototype.render = function () {
        var _a = this.props, data = _a.data, value = _a.value, zoom = _a.zoom;
        var rect = this.context.rectToPage(data.pageindex, data.rect);
        var width = rect.right - rect.left;
        var height = Math.abs(rect.bottom - rect.top);
        var style = {
            top: 0,
            position: "absolute",
            left: 0,
            width: width + "px",
            height: height + "px",
            boxSizing: "border-box",
            display: data.flags == WidgetFlags.Hidden ? "none" : "block"
        };
        var fontSize = 0;
        if (data.defaultAppearance.size !== 0) {
            fontSize = Math.floor(data.defaultAppearance.size * getPixelsPerPoint() * zoom);
        }
        else {
            fontSize = Math.floor(height * 0.55 * getPixelsPerPoint() * zoom);
        }
        var inputStyle = {
            position: "absolute",
            width: width + "px",
            maxWidth: width,
            height: height,
            maxHeight: height,
            textAlign: this.getAlignment(data.alignment),
            padding: 0,
            border: 0,
            background: data.focus ? "#fff" : WidgetBase.highLightColor,
            fontSize: fontSize,
            zIndex: 4,
            //outline: data.focus ? "#ddd double thin" : 0,
            outline: 0,
            paddingLeft: "1px",
            color: parseColor(data.defaultAppearance.color),
        };
        if (data.defaultAppearance.size != 0) {
            inputStyle.fontFamily = data.defaultAppearance.font;
        }
        if (data.fieldFlags == FieldFlags.FlagTextMultiline) {
            inputStyle.resize = "none";
        }
        else {
            inputStyle.lineHeight = height;
        }
        translateDOMPositionXY(style, rect.left + 1, rect.top);
        var showValue = value;
        var formatAction;
        if (!data.focus) {
            showValue = this.formateViewValue(value);
        }
        var readOnly = data.fieldFlags == FieldFlags.FlagReadOnly;
        return (h("div", { "data-type": "text", style: style, "data-name": data.name }, data.fieldFlags == FieldFlags.FlagTextMultiline ?
            h("textarea", { type: "text", class: "fwr-form-widget", readOnly: readOnly, name: data.name, style: inputStyle, onFocus: this.onFocus, onBlur: this.onBlur, value: showValue, onChange: this.onChange }) :
            h("input", { type: "text", class: "fwr-form-widget", readOnly: readOnly, name: data.name, style: inputStyle, onFocus: this.onFocus, onBlur: this.onBlur, value: showValue, onChange: this.onChange })));
    };
    __decorate([
        bind
    ], WidgetText.prototype, "onFocus", null);
    __decorate([
        bind
    ], WidgetText.prototype, "onBlur", null);
    __decorate([
        bind
    ], WidgetText.prototype, "onChange", null);
    return WidgetText;
}(WidgetBase));
export default WidgetText;
//# sourceMappingURL=WidgetText.js.map