import UserAgent_DEPRECATED from "../vendor/UserAgent_DEPRECATED";
import AsyncRetryExecutor from "../vendor/AsyncRetryExecutor";
export var TILE_SIZE = 400;
function blobToURL(blobFile, imageUrl) {
    if (UserAgent_DEPRECATED.ie() < 10) {
        return imageUrl;
    }
    return window.URL.createObjectURL(blobFile);
}
function loadImage(url) {
    return new Promise(function (resolve, reject) {
        if (!url)
            return reject();
        var xhr = new XMLHttpRequest();
        xhr.open('get', url, true);
        xhr.responseType = 'blob';
        xhr.onload = function () {
            var _this = this;
            if (_this.status == 200) {
                resolve(_this.response);
            }
            else {
                reject(_this.status);
            }
        };
        xhr.send();
    })
        .then(function (blob) {
        return blobToURL(blob, url);
    });
}
function generatorKey(_a) {
    var docId = _a.docId, pageIndex = _a.pageIndex, colIndex = _a.colIndex, rowIndex = _a.rowIndex, zoomName = _a.zoomName, imageWithAnnot = _a.imageWithAnnot, imageWithForm = _a.imageWithForm;
    return docId + ":" + pageIndex + ":" + rowIndex + ":" + colIndex + ":" + zoomName + ":" + imageWithAnnot + ":" + imageWithForm;
}
var ImageUrlCache = /** @class */ (function () {
    function ImageUrlCache() {
    }
    ImageUrlCache.get = function (_a) {
        var docId = _a.docId, pageIndex = _a.pageIndex, colIndex = _a.colIndex, rowIndex = _a.rowIndex, zoomName = _a.zoomName, imageWithAnnot = _a.imageWithAnnot, imageWithForm = _a.imageWithForm;
        var key = generatorKey({ docId: docId, pageIndex: pageIndex, colIndex: colIndex, rowIndex: rowIndex, zoomName: zoomName, imageWithAnnot: imageWithAnnot, imageWithForm: imageWithForm });
        if (ImageUrlCache.urlCache.hasOwnProperty(key)) {
            return ImageUrlCache.urlCache[key];
        }
        else {
            return null;
        }
    };
    ImageUrlCache.set = function (_a, url) {
        var docId = _a.docId, pageIndex = _a.pageIndex, colIndex = _a.colIndex, rowIndex = _a.rowIndex, zoomName = _a.zoomName, imageWithAnnot = _a.imageWithAnnot, imageWithForm = _a.imageWithForm;
        var key = generatorKey({ docId: docId, pageIndex: pageIndex, colIndex: colIndex, rowIndex: rowIndex, zoomName: zoomName, imageWithAnnot: imageWithAnnot, imageWithForm: imageWithForm });
        ImageUrlCache.urlCache[key] = url;
    };
    ImageUrlCache.urlCache = {};
    return ImageUrlCache;
}());
function createImgUrl(_a) {
    var pageIndex = _a.pageIndex, zoomName = _a.zoomName, tileSize = _a.tileSize, rowIndex = _a.rowIndex, colIndex = _a.colIndex, manifestInfo = _a.manifestInfo, docId = _a.docId, token = _a.token, serverUrl = _a.serverUrl, autoParse = _a.autoParse, imageWithAnnot = _a.imageWithAnnot, imageWithForm = _a.imageWithForm;
    if (manifestInfo.SecurityType == 1) {
        return "/api/v2/parser/" + docId + "/image/" + manifestInfo.ConnectedInfo.docId + "/" + pageIndex + "/tiles/" + zoomName + "/" + tileSize + "/" + colIndex + "/" + rowIndex + "?autoParse=" + autoParse + "&access-token=" + token + "&imageWithAnnot=" + imageWithAnnot + "&imageWithForm=" + imageWithForm;
    }
    else {
        return serverUrl + "/api/v2/parser/" + docId + "/image/" + pageIndex + "/tiles/" + zoomName + "/" + tileSize + "/" + colIndex + "/" + rowIndex + "?autoParse=" + autoParse + "&imageWithAnnot=" + imageWithAnnot + "&imageWithForm=" + imageWithForm;
    }
}
export function getImageUrl(_a) {
    var docId = _a.docId, pageIndex = _a.pageIndex, colIndex = _a.colIndex, rowIndex = _a.rowIndex, zoomName = _a.zoomName, imageWithAnnot = _a.imageWithAnnot, imageWithForm = _a.imageWithForm;
    return ImageUrlCache.get({ docId: docId, pageIndex: pageIndex, colIndex: colIndex, rowIndex: rowIndex, zoomName: zoomName, imageWithAnnot: imageWithAnnot, imageWithForm: imageWithForm });
}
export function loadTileImage(_a, success, fail) {
    var pageIndex = _a.pageIndex, rowIndex = _a.rowIndex, colIndex = _a.colIndex, zoomName = _a.zoomName, tileSize = _a.tileSize, docId = _a.docId, token = _a.token, manifestInfo = _a.manifestInfo, autoParse = _a.autoParse, imageWithForm = _a.imageWithForm, imageWithAnnot = _a.imageWithAnnot, serverUrl = _a.serverUrl;
    var url = ImageUrlCache.get({ docId: docId, pageIndex: pageIndex, colIndex: colIndex, rowIndex: rowIndex, zoomName: zoomName, imageWithAnnot: imageWithAnnot, imageWithForm: imageWithForm });
    if (typeof url === "string") {
        success(url);
        return;
    }
    if (url && url.then) {
        url.then(function (url) {
            success(url);
        }).catch(function (error) {
            fail && fail();
        });
        return;
    }
    var imageUrl = createImgUrl({
        pageIndex: pageIndex,
        zoomName: zoomName,
        tileSize: tileSize,
        rowIndex: rowIndex,
        colIndex: colIndex,
        docId: docId,
        manifestInfo: manifestInfo,
        token: token,
        serverUrl: serverUrl,
        autoParse: autoParse,
        imageWithAnnot: imageWithAnnot,
        imageWithForm: imageWithForm
    });
    var executor = new AsyncRetryExecutor({ maxRetry: 100, delay: 1500 });
    var promise = executor.getWithRetry(function () {
        return loadImage(imageUrl);
    });
    promise.then(function (url) {
        if (url) {
            ImageUrlCache.set({ docId: docId, pageIndex: pageIndex, colIndex: colIndex, rowIndex: rowIndex, zoomName: zoomName, imageWithAnnot: imageWithAnnot, imageWithForm: imageWithForm }, url);
            success(url);
            return url;
        }
        else {
            ImageUrlCache.set({ docId: docId, pageIndex: pageIndex, colIndex: colIndex, rowIndex: rowIndex, zoomName: zoomName, imageWithAnnot: imageWithAnnot, imageWithForm: imageWithForm }, null);
            fail && fail();
        }
    })
        .catch(function (error) {
        //canceled
        console.log("error", error);
        ImageUrlCache.set({ docId: docId, pageIndex: pageIndex, colIndex: colIndex, rowIndex: rowIndex, zoomName: zoomName, imageWithAnnot: imageWithAnnot, imageWithForm: imageWithForm }, null);
        fail && fail();
    });
    ImageUrlCache.set({ docId: docId, pageIndex: pageIndex, colIndex: colIndex, rowIndex: rowIndex, zoomName: zoomName, imageWithAnnot: imageWithAnnot, imageWithForm: imageWithForm }, promise);
    return function cancel() {
        executor.dontRetry();
        fail && fail();
    };
}
//# sourceMappingURL=loadTileImage.js.map