var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { h, Component } from "preact";
import i18next from "i18next";
import Rnd from "../Rnd";
import bind from "../../vendor/bind";
import Size from "../../utils/size";
import * as query from "dom-helpers/query";
import translateDOMPositionXY from "../../vendor/translateDOMPositionXY";
import EventListener from "../../vendor/EventListener";
import classNames from "classnames";
import { SignType } from "../../sign/models";
import * as shallowEqual from "shallow-equals";
import UserAgent from "../../vendor/UserAgent_DEPRECATED";
import ContextMenuTrigger from "../ContextMenuTrigger";
var SignField = /** @class */ (function (_super) {
    __extends(SignField, _super);
    function SignField() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SignField.prototype.componentDidMount = function () {
        this._mouseDownToken = EventListener.listen(this.base, "mousedown", this.onMouseDown);
        if (UserAgent.mobile()) {
            this._touchStartToken = EventListener.listen(this.base, "touchstart", this.onTouchStart);
        }
    };
    SignField.prototype.componentWillUnmount = function () {
        this._mouseDownToken && this._mouseDownToken.remove();
        this._touchStartToken && this._touchStartToken.remove();
    };
    SignField.prototype.shouldComponentUpdate = function (nextProps, nextState) {
        return !shallowEqual(this.state, nextState) || !shallowEqual(this.props, nextProps);
    };
    SignField.prototype.signFieldResize = function (direction, elementRef, delta, position) {
        var fieldMeta = this.props.fieldMeta;
        var offset = query.offset(elementRef);
        var size = new Size(this.context.lengthToPDF(offset.width), this.context.lengthToPDF(offset.height));
        this.context.updateSignField(fieldMeta, { size: size, pos: this.context.ptToPDF(fieldMeta.pageIndex, position) }, false);
    };
    SignField.prototype.signFieldResizeStop = function (direction, elementRef, delta, position) {
        var fieldMeta = this.props.fieldMeta;
        var offset = query.offset(elementRef);
        var size = new Size(this.context.lengthToPDF(offset.width), this.context.lengthToPDF(offset.height));
        this.context.updateSignField(fieldMeta, { size: size, pos: this.context.ptToPDF(fieldMeta.pageIndex, position) }, true);
    };
    SignField.prototype.signFieldDragStop = function (event, dragData) {
        // event.stopPropagation();
        this.context.signFieldDragStop(this.props.fieldMeta, dragData);
    };
    SignField.prototype.signFieldDragStart = function (event, dragData) {
        //event.stopPropagation();
        this.context.signFieldDragStart(event, dragData);
    };
    SignField.prototype.onMouseDown = function (event) {
        this.context.activeSignField(this.props.fieldMeta);
        // event.stopPropagation();
    };
    SignField.prototype.onTouchStart = function (event) {
        this.context.activeSignField(this.props.fieldMeta);
    };
    SignField.prototype.getTitle = function (type) {
        return type == SignType.date_signed ? "date" : type;
    };
    SignField.prototype.getSignFieldIcon = function (type) {
        return this.context.getSignFieldIcon(type);
    };
    SignField.prototype.keepRatio = function (imgWidth, imgHeight, ratio) {
        if (imgWidth / imgHeight > ratio) {
            imgWidth = imgHeight * ratio;
        }
        else {
            imgHeight = imgWidth / ratio;
        }
        return { imgWidth: imgWidth, imgHeight: imgHeight };
    };
    SignField.prototype.getMinWidth = function (signType, pdfWidth) {
        if (signType == SignType.signature) {
            return Math.max(100, this.context.lengthToPage(pdfWidth));
        }
        if (signType == SignType.initials) {
            return Math.max(82, this.context.lengthToPage(pdfWidth));
        }
        if (signType == SignType.date_signed) {
            return Math.max(72, this.context.lengthToPage(pdfWidth));
        }
    };
    SignField.prototype.getMinHeight = function (signType, pdfHeight) {
        return Math.max(28, this.context.lengthToPage(pdfHeight));
    };
    SignField.prototype.render = function () {
        var _this = this;
        var _a = this.props, signing = _a.signing, rndable = _a.rndable, fieldMeta = _a.fieldMeta, curSignField = _a.curSignField;
        var pos = fieldMeta.pos, size = fieldMeta.size, active = fieldMeta.active, type = fieldMeta.type, id = fieldMeta.id, signerId = fieldMeta.signerId, signed = fieldMeta.signed, signData = fieldMeta.signData, pageIndex = fieldMeta.pageIndex;
        var pagePt = this.context.pdfToPage(pageIndex, pos);
        var width = this.getMinWidth(type, size.width);
        var height = this.getMinHeight(type, size.height);
        var colorScheme = this.context.recipientColorScheme(signerId, 0.3);
        if (signed) {
            var signStyle = {
                top: 0,
                position: "absolute",
                left: 0,
                width: width + "px",
                height: height + "px",
                boxSizing: "border-box",
                zIndex: 0
            };
            var otherProps = {};
            if (signing) {
                otherProps["onClick"] = function (event) {
                    _this.context.updateSignField(fieldMeta, { signed: false }, false);
                };
            }
            translateDOMPositionXY(signStyle, pagePt.x, pagePt.y);
            var _b = this.keepRatio(width, height, signData.width / signData.height), imgWidth = _b.imgWidth, imgHeight = _b.imgHeight;
            return (h("div", __assign({ style: signStyle }, otherProps),
                h("img", { src: signData.dataurl, width: imgWidth, height: imgHeight })));
        }
        if (!rndable) {
            var signStyle = {
                top: 0,
                position: "absolute",
                left: 0,
                width: width + "px",
                height: height + "px",
                boxSizing: "border-box",
                pointerEvents: signing ? "all" : "none",
                zIndex: 1
            };
            translateDOMPositionXY(signStyle, pagePt.x, pagePt.y);
            var otherProps = {};
            if (signing) {
                otherProps["onClick"] = function (event) {
                    _this.context.startSign(_this.props.fieldMeta);
                    event.stopPropagation();
                };
            }
            return (h("div", __assign({ id: id, class: classNames("sign_field", { sign_signing: signing, cur_sign_field: curSignField }), style: Object.assign(signStyle, colorScheme) }, otherProps),
                h("div", { class: "icon" },
                    h("img", { src: this.getSignFieldIcon(type) })),
                h("div", { class: "title" },
                    h("span", null, i18next.t(this.getTitle(type))))));
        }
        else {
            var enableResizing = active && {
                bottomLeft: true,
                bottomRight: true,
                topLeft: true,
                topRight: true
            };
            var handleStyle = {
                borderColor: colorScheme.borderColor,
            };
            var handleStyles = {
                topRight: handleStyle,
                bottomRight: handleStyle,
                bottomLeft: handleStyle,
                topLeft: handleStyle
            };
            return (h(Rnd, { bounds: "#" + this.context.docViewId, position: {
                    x: pagePt.x,
                    y: pagePt.y
                }, size: {
                    width: width,
                    height: height
                }, enableResizing: enableResizing, minHeight: this.getMinHeight(type, 27), minWidth: this.getMinWidth(type, 128), maxHeight: this.context.lengthToPage(54), maxWidth: this.context.lengthToPage(228), resizeHandleStyles: handleStyles, style: { display: "flex" }, onDragStart: this.signFieldDragStart, onDragStop: this.signFieldDragStop, onResize: this.signFieldResize, onResizeStop: this.signFieldResizeStop },
                h(ContextMenuTrigger, { target: "#sign_page_context_menu", disable: !UserAgent.mobile(), showTop: true, holdToDisplay: 1000 },
                    h("div", { id: id, class: "sign_field", style: Object.assign({ flex: "1 1 auto" }, colorScheme) },
                        h("div", { class: "icon" },
                            h("img", { src: this.getSignFieldIcon(type) })),
                        h("div", { class: "title" },
                            h("span", null, i18next.t(this.getTitle(type))))))));
        }
    };
    __decorate([
        bind
    ], SignField.prototype, "signFieldResize", null);
    __decorate([
        bind
    ], SignField.prototype, "signFieldResizeStop", null);
    __decorate([
        bind
    ], SignField.prototype, "signFieldDragStop", null);
    __decorate([
        bind
    ], SignField.prototype, "signFieldDragStart", null);
    __decorate([
        bind
    ], SignField.prototype, "onMouseDown", null);
    __decorate([
        bind
    ], SignField.prototype, "onTouchStart", null);
    return SignField;
}(Component));
export default SignField;
//# sourceMappingURL=SignField.js.map