var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { h } from "preact";
import bind from "../vendor/bind";
import FormPage from "../components/form/FormPage";
import { FormWidgetType, Widget, ActionTrigger } from "./models";
import * as actions from "./actions";
import { Map } from "immutable";
import reducer from "./FormPluginReducer";
import { EventEmitter } from "events";
import RunPDFJS from "../form/ActionJScript";
require("../style/form.less");
var ARG_ARRAY = /\s*new Array\s*[^\(]*\(\s*([^\)]*)\)/m;
var ARG_FIELD = /\s*getField\s*[^\(]*\(\s*([^\)]*)\)/mg;
function makeEventName(str) {
    return str.replace(/(^\s*)|(\s*$)/g, '');
}
var FormPlugin = /** @class */ (function () {
    function FormPlugin(store, options) {
        this._event = new EventEmitter();
        this.name = "form";
        this.reducer = reducer;
        this._loadedFormFields = false;
        this._store = store;
        this._options = options;
        this._jsEngine = RunPDFJS(this.setItemValue, this.getItemValue);
    }
    FormPlugin.prototype.getFormValues = function () {
        var values = [];
        this.webpdf.getState().form.get("formItemValues", Map())
            .filter(function (value, name) {
            return value.get("origValue") != value.get("value");
        }).forEach(function (value, name) {
            values.push({ name: name, value: value.get("value") });
        });
        return values;
    };
    FormPlugin.prototype.createWiget = function (widget, exportValue, type, name, fieldFlags, defaultAppearance, alignment) {
        return new Widget(Object.assign({ type: type, name: name, exportValue: exportValue, fieldFlags: fieldFlags, defaultAppearance: defaultAppearance, alignment: alignment }, widget));
    };
    FormPlugin.prototype.getFormFields = function (serverUrl, docId) {
        var url = serverUrl + "/api/v2/parser/" + docId + "/form/fields";
        var _self = this;
        return new Promise(function (resolve, reject) {
            var xhr = new XMLHttpRequest();
            xhr.open('get', url, true);
            xhr.responseType = 'text';
            xhr.onload = function () {
                var _this = this;
                if (_this.status == 200) {
                    resolve(JSON.parse(_this.response));
                }
                else {
                    reject(_this.status);
                }
            };
            xhr.send();
        })
            .then(function (fields) {
            if (fields.length) {
                var widgets_1 = [];
                var formItemValues_1 = {};
                fields.forEach(function (field) {
                    var name = field.name, type = field.type, value = field.value, flags = field.flags, defaultAppearance = field.defaultAppearance, additionalAction = field.additionalAction, alignment = field.alignment;
                    field.control.forEach(function (control) {
                        var widget = _self.createWiget(control.widget, control.exportValue, type, name, flags, defaultAppearance, alignment);
                        additionalAction && _self.getRegisterActionCode(widget, additionalAction);
                        widgets_1.push(widget);
                    });
                    formItemValues_1[name] = { origValue: value, value: value };
                });
                _self.webpdf.getStore().dispatch(actions.setFormItemValues(formItemValues_1));
                _self.webpdf.getStore().dispatch(actions.setWidgets(widgets_1));
            }
        });
    };
    FormPlugin.prototype.getRegisterActionCode = function (widget, additionalAction) {
        var _this = this;
        if (widget.type == FormWidgetType.Text) {
            additionalAction.forEach(function (action) {
                _this.addAction(action, widget.name, widget.formatters, widget.validater);
            });
        }
    };
    FormPlugin.prototype.addAction = function (action, widgetName, formatters, validater) {
        var script = action.script.replace(/\/\*{1,2}[\s\S]*?\*\//, "");
        var _self = this;
        if (action.trigger == ActionTrigger.TriggerFieldRecalculateValue) {
            var itemNames = [];
            var argDecl = script.match(ARG_ARRAY);
            if (argDecl && argDecl.length > 1) {
                itemNames = argDecl[1].replace(/\"/g, "").split(",");
            }
            else {
                argDecl = ARG_FIELD.exec(script);
                while (argDecl) {
                    itemNames.push(argDecl[1].replace(/\"/g, ""));
                    argDecl = ARG_FIELD.exec(script);
                }
            }
            this.onItemValueChange(itemNames, function () {
                var ret = _self._jsEngine(action.script);
                if (ret !== null) {
                    _self.setItemValue(widgetName, ret.value);
                }
            });
        }
        if (action.trigger == ActionTrigger.TriggerFieldWillFormat) {
            formatters.push(function (srcValue) {
                return _self._jsEngine(action.script, null, srcValue).value;
            });
        }
        if (action.trigger == ActionTrigger.TriggerFieldValueChanged) {
            validater.push(function (srcValue, element) {
                return _self._jsEngine(action.script, element, srcValue).rc !== false;
            });
        }
    };
    FormPlugin.prototype.onItemValueChange = function (itemNames, func) {
        var _this = this;
        if (typeof itemNames.length != "number") {
            itemNames = [itemNames];
        }
        itemNames.forEach(function (itemName) {
            _this._event.on(makeEventName(itemName), func);
        });
    };
    FormPlugin.prototype.setItemValue = function (name, value) {
        this.webpdf.getStore().dispatch(actions.setItemValue(name, value));
        this._event.emit(name);
    };
    FormPlugin.prototype.getItemValue = function (name) {
        return this.webpdf.getState().form.getIn(["formItemValues", name, "value"]);
    };
    FormPlugin.prototype.renderPage = function (_a) {
        var pageIndex = _a.pageIndex, zoom = _a.zoom;
        return h(FormPage, { pageIndex: pageIndex, setItemValue: this.setItemValue, zoom: zoom });
    };
    FormPlugin.prototype.renderDoc = function (props) {
        if (!this._loadedFormFields) {
            this._loadedFormFields = true;
            this.getFormFields(props.serverUrl, props.docId);
        }
        return null;
    };
    __decorate([
        bind
    ], FormPlugin.prototype, "onItemValueChange", null);
    __decorate([
        bind
    ], FormPlugin.prototype, "setItemValue", null);
    __decorate([
        bind
    ], FormPlugin.prototype, "getItemValue", null);
    return FormPlugin;
}());
export default FormPlugin;
//# sourceMappingURL=FormPlugin.js.map