var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { h } from "preact";
import bind from "../vendor/bind";
import SignPage from "../components/sign/SignPage";
import NavigateBtn from "../components/sign/NavigateBtn";
import * as actions from "./actions";
import { Map } from "immutable";
import reducer from "./SignPluginReducer";
import ContextMenu from "../components/sign/ContextMenu";
import createDateSignData from "../utils/createDateSignData";
import createDigitalSign from "../utils/createDigitalSign";
import { getUserInfo } from "../selector";
var SignPlugin = /** @class */ (function () {
    function SignPlugin(store, options) {
        this.name = "sign";
        this.reducer = reducer;
        this._store = store;
        this._options = options;
    }
    SignPlugin.prototype.onMouseEnterPage = function () {
        this._store.dispatch(actions.showFloatIcon(false));
        this._store.dispatch(actions.setMouseInPage(true));
    };
    SignPlugin.prototype.onMouseLeavePage = function () {
        this._store.dispatch(actions.setMouseInPage(false));
    };
    SignPlugin.prototype.getSelectedFieldType = function () {
        return this._store.getState().sign.selectedFieldType;
    };
    SignPlugin.prototype.getSignFieldIcon = function (type) {
        return this._options.signFieldIcons[type];
    };
    SignPlugin.prototype.getSelectedRecipient = function () {
        return this._store.getState().sign.selectedRecipient;
    };
    SignPlugin.prototype.getRecipients = function () {
        return this._store.getState().sign.recipients;
    };
    SignPlugin.prototype.getUserInfo = function () {
        return getUserInfo(this._store.getState());
    };
    /**获取用户设置的签名图片 */
    SignPlugin.prototype.getSignDatas = function (signField) {
        var _self = this;
        var userId = this.getUserInfo().user_id;
        if (signField.type == "date_signed") {
            var signData = createDateSignData(signField.size.width, signField.size.height);
            return createDigitalSign(signData, signField.size, userId);
        }
        else {
            var signDatas_1 = this._store.getState().sign.signDatas;
            if (!signDatas_1[signField.type]) {
                if (_self.setSignDatas) {
                    return new Promise(function (resovle) {
                        _self.setSignDatas(signField.type);
                        _self.setSignData = function (type, signData) {
                            resovle(signData);
                        };
                    }).then(function (signData) {
                        return createDigitalSign(signDatas_1[signField.type], signField.size, userId);
                    });
                }
                else {
                    throw "setSignDatas func is no set";
                }
            }
            else {
                return createDigitalSign(signDatas_1[signField.type], signField.size, userId);
            }
        }
    };
    SignPlugin.prototype.onAddSignField = function () {
        this._store.dispatch(actions.setSelectedFieldType(null));
        this._store.dispatch(actions.showFloatIcon(false));
    };
    SignPlugin.prototype.addSignField = function (signField) {
        this.webpdf.getStore().dispatch(actions.addSignField(signField, true));
    };
    SignPlugin.prototype.getSignFieldList = function () {
        return this.webpdf.getState().sign.present.get("fields", Map()).toList();
    };
    SignPlugin.prototype.getSignFields = function () {
        return this.webpdf.getState().sign.present.get("fields", Map());
    };
    SignPlugin.prototype.getCurSignField = function () {
        return this.webpdf.getState().sign.present.get("curSignField", null);
    };
    SignPlugin.prototype.getRecipientColorScheme = function (recipient, opacity) {
        return this._options.getRecipientColorScheme(recipient, opacity);
    };
    SignPlugin.prototype.renderPage = function (_a) {
        var pageIndex = _a.pageIndex, zoom = _a.zoom;
        return h(SignPage, { getSelectedFieldType: this.getSelectedFieldType, getSelectedRecipient: this.getSelectedRecipient, getSignFieldIcon: this.getSignFieldIcon, getRecipients: this.getRecipients, getSignDatas: this.getSignDatas, onMouseEnter: this.onMouseEnterPage, onMouseLeave: this.onMouseLeavePage, onAddSignField: this.onAddSignField, getRecipientColorScheme: this.getRecipientColorScheme, pageIndex: pageIndex, signing: this._options.signing, zoom: zoom });
    };
    SignPlugin.prototype.renderDoc = function (props) {
        if (this._options.signing) {
            return h(NavigateBtn, __assign({}, props, { selectedRecipient: this.getSelectedRecipient() }));
        }
        else {
            return h(ContextMenu, null);
        }
    };
    __decorate([
        bind
    ], SignPlugin.prototype, "onMouseEnterPage", null);
    __decorate([
        bind
    ], SignPlugin.prototype, "onMouseLeavePage", null);
    __decorate([
        bind
    ], SignPlugin.prototype, "getSelectedFieldType", null);
    __decorate([
        bind
    ], SignPlugin.prototype, "getSignFieldIcon", null);
    __decorate([
        bind
    ], SignPlugin.prototype, "getSelectedRecipient", null);
    __decorate([
        bind
    ], SignPlugin.prototype, "getRecipients", null);
    __decorate([
        bind
    ], SignPlugin.prototype, "getUserInfo", null);
    __decorate([
        bind
    ], SignPlugin.prototype, "getSignDatas", null);
    __decorate([
        bind
    ], SignPlugin.prototype, "onAddSignField", null);
    __decorate([
        bind
    ], SignPlugin.prototype, "getRecipientColorScheme", null);
    return SignPlugin;
}());
export default SignPlugin;
//# sourceMappingURL=SignPlugin.js.map