var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import EventListener from "./EventListener";
import getEventClientPos from "./getPtClientPos";
import UserAgent from "./UserAgent_DEPRECATED";
import bind from './bind';
var DOMMouseMoveTracker = /** @class */ (function () {
    function DOMMouseMoveTracker(onMove, onMoveEnd, domNode) {
        this._isDragging = false;
        this._animationFrameID = null;
        this._domNode = domNode;
        this._onMove = onMove;
        this._onMoveEnd = onMoveEnd;
    }
    /**
     * This is to set up the listeners for listening to mouse move
     * and mouse up signaling the movement has ended. Please note that these
     * listeners are added at the document.body level. It takes in an event
     * in order to grab inital state.
     */
    DOMMouseMoveTracker.prototype.captureMouseMoves = function (event) {
        if (!this._eventMoveToken && !this._eventUpToken && !this._eventTouchMoveToken && !this._eventTouchEndToken) {
            this._eventMoveToken = EventListener.listen(this._domNode, 'mousemove', this._onMouseMove);
            this._eventUpToken = EventListener.listen(this._domNode, 'mouseup', this._onMouseUp);
            if (UserAgent.mobile()) {
                this._eventTouchMoveToken = EventListener.listen(this._domNode, 'touchmove', this._onMouseMove);
                this._eventTouchEndToken = EventListener.listen(this._domNode, 'touchend', this._onMouseUp);
            }
        }
        if (!this._isDragging) {
            this._deltaX = 0;
            this._deltaY = 0;
            this._isDragging = true;
            var pos = getEventClientPos(event);
            this._x = pos.clientX;
            this._y = pos.clientY;
        }
        event.type == "mouseup" && event.preventDefault();
    };
    /**
     * These releases all of the listeners on document.body.
     */
    DOMMouseMoveTracker.prototype.releaseMouseMoves = function () {
        if (this._eventMoveToken) {
            this._eventMoveToken.remove();
            this._eventMoveToken = null;
        }
        if (this._eventUpToken) {
            this._eventUpToken.remove();
            this._eventUpToken = null;
        }
        if (this._eventTouchMoveToken) {
            this._eventTouchMoveToken.remove();
            this._eventTouchMoveToken = null;
        }
        if (this._eventTouchEndToken) {
            this._eventTouchEndToken.remove();
            this._eventTouchEndToken = null;
        }
        if (this._animationFrameID !== null) {
            cancelAnimationFrame(this._animationFrameID);
            this._animationFrameID = null;
        }
        if (this._isDragging) {
            this._isDragging = false;
            this._x = null;
            this._y = null;
        }
    };
    /**
     * Returns whether or not if the mouse movement is being tracked.
     */
    DOMMouseMoveTracker.prototype.isDragging = function () {
        return this._isDragging;
    };
    /**
     * Calls onMove passed into constructor and updates internal state.
     */
    DOMMouseMoveTracker.prototype._onMouseMove = function (event) {
        if (event.type == "touchmove" && event.touches.length > 1) {
            return;
        }
        var pos = getEventClientPos(event);
        var x = pos.clientX;
        var y = pos.clientY;
        this._deltaX += (x - this._x);
        this._deltaY += (y - this._y);
        if (this._animationFrameID === null) {
            // The mouse may move faster then the animation frame does.
            // Use `requestAnimationFramePolyfill` to avoid over-updating.   
            this._animationFrameID = requestAnimationFrame(this._didMouseMove);
        }
        this._x = x;
        this._y = y;
    };
    DOMMouseMoveTracker.prototype._didMouseMove = function () {
        this._animationFrameID = null;
        this._onMove(this._deltaX, this._deltaY, { clientX: this._x, clientY: this._y });
        this._deltaX = 0;
        this._deltaY = 0;
    };
    /**
     * Calls onMoveEnd passed into constructor and updates internal state.
     */
    DOMMouseMoveTracker.prototype._onMouseUp = function (event) {
        if (this._animationFrameID) {
            this._didMouseMove();
        }
        this._onMoveEnd(this._x, this._y);
    };
    __decorate([
        bind
    ], DOMMouseMoveTracker.prototype, "_onMouseMove", null);
    __decorate([
        bind
    ], DOMMouseMoveTracker.prototype, "_didMouseMove", null);
    __decorate([
        bind
    ], DOMMouseMoveTracker.prototype, "_onMouseUp", null);
    return DOMMouseMoveTracker;
}());
export default DOMMouseMoveTracker;
//# sourceMappingURL=DOMMouseMoveTracker.js.map