/**
 * Upstream version of event listener. Does not take into account specific
 * nature of platform.
 */
import UserAgent from "./UserAgent_DEPRECATED";
var EventListener = {
    /**
     * Listen to DOM events during the bubble phase.
     *
     * @param {DOMEventTarget} target DOM element to register listener on.
     * @param {string} eventType Event type, e.g. 'click' or 'mouseover'.
     * @param {function} callback Callback function.
     * @return {object} Object with a `remove` method.
     */
    listen: function listen(target, eventType, callback) {
        var options = UserAgent.chrome() || UserAgent.webkit() ? { passive: false } : false;
        if (target.addEventListener) {
            target.addEventListener(eventType, callback, options);
            return {
                remove: function remove() {
                    target.removeEventListener(eventType, callback, options);
                }
            };
        }
        else if (target.attachEvent) {
            target.attachEvent('on' + eventType, callback);
            return {
                remove: function remove() {
                    target.detachEvent('on' + eventType, callback);
                }
            };
        }
    }
};
export default EventListener;
//   /**
//    * Listen to DOM events during the capture phase.
//    *
//    * @param {DOMEventTarget} target DOM element to register listener on.
//    * @param {string} eventType Event type, e.g. 'click' or 'mouseover'.
//    * @param {function} callback Callback function.
//    * @return {object} Object with a `remove` method.
//    */
//   capture: function capture(target, eventType, callback) {
//     if (target.addEventListener) {
//       target.addEventListener(eventType, callback, true);
//       return {
//         remove: function remove() {
//           target.removeEventListener(eventType, callback, true);
//         }
//       };
//     } else {
//       if (process.env.NODE_ENV !== 'production') {
//         console.error('Attempted to listen to events during the capture phase on a ' + 'browser that does not support the capture phase. Your application ' + 'will not receive some events.');
//       }
//       return {
//         remove: emptyFunction
//       };
//     }
//   },
//   registerDefault: function registerDefault() { }
// };
// module.exports = EventListener;
//# sourceMappingURL=EventListener.js.map