var pathSeparator = '/';
function join() {
    var paths = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        paths[_i] = arguments[_i];
    }
    var path = '';
    for (var i = 0, j = paths.length; i < j; i++) {
        var segment = paths[i];
        if (segment) {
            if (!path) {
                path += segment;
            }
            else {
                path += pathSeparator + segment;
            }
        }
    }
    return normalize(path);
}
function isAbsolute(path) {
    return path[0] === pathSeparator;
}
function urlJoin(url) {
    var paths = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        paths[_i - 1] = arguments[_i];
    }
    var parsedUrl = parseUrl(url);
    paths.unshift(popPath(parsedUrl.pathname));
    return parsedUrl.protocol + '://' + parsedUrl.host + join.apply(void 0, paths) + parsedUrl.search + parsedUrl.hash;
}
function normalize(path) {
    var _isAbsolute = isAbsolute(path);
    var trailingSlash = path && path[path.length - 1] === pathSeparator;
    path = normalizeArray(path.split(pathSeparator), !isAbsolute).join(pathSeparator);
    if (!path && !isAbsolute) {
        return '';
    }
    if (path && trailingSlash) {
        path += pathSeparator;
    }
    return (_isAbsolute ? pathSeparator : '') + path;
}
function normalizeArray(parts, allowAboveRoot) {
    var res = [];
    for (var i = 0, j = parts.length; i < j; i++) {
        var p = parts[i];
        if (!p || p === '.') {
            continue;
        }
        if (p === '..') {
            if (res.length && res[res.length - 1] !== '..') {
                res.pop();
            }
            else if (allowAboveRoot) {
                res.push('..');
            }
        }
        else {
            res.push(p);
        }
    }
    return res;
}
function isUrl(url) {
    return urlReg().test(url);
}
function urlReg() {
    return /^([A-z]+)\:\/\/([A-z0-9\-_\.]+)(\:(\d+))?(\/[^?#]*)?(\?[^#]*)?(#.*)?$/;
}
function parseUrl(url) {
    var reg = urlReg();
    var match = url.match(reg);
    if (!match) {
        throw 'this is not a valid url';
    }
    return {
        match: match,
        protocol: match[1],
        hostname: match[2],
        host: match[2] + (match[3] || ''),
        port: +match[4] || 80,
        pathname: match[5] || '/',
        search: match[6] || '',
        hash: match[7] || ''
    };
}
function getCurrentPath() {
    var url = parseUrl(location.href);
    return url.protocol + '://' + url.host + popPath(url.pathname);
}
function popPath(path) {
    return path.split(pathSeparator).slice(0, -1).join(pathSeparator) || '/';
}
export { getCurrentPath, join, isUrl, urlJoin, };
//# sourceMappingURL=path.js.map