export function parseActions(rawActions, defaultValue) {
    if (defaultValue === void 0) { defaultValue = []; }
    if (Array.isArray(rawActions)) {
        return rawActions;
    }
    else if (typeof rawActions === 'string') {
        return [rawActions];
    }
    return defaultValue;
}
// isHistory helper: check for a valid history object
export function isHistory(history) {
    return typeof history.present !== 'undefined' &&
        typeof history.future !== 'undefined' &&
        typeof history.past !== 'undefined' &&
        Array.isArray(history.future) &&
        Array.isArray(history.past);
}
// includeAction helper: whitelist actions to be added to the history
export function includeAction(rawActions) {
    var actions = parseActions(rawActions);
    return function (action) { return actions.indexOf(action.type) >= 0; };
}
// excludeAction helper: blacklist actions from being added to the history
export function excludeAction(rawActions) {
    var actions = parseActions(rawActions);
    return function (action) { return actions.indexOf(action.type) < 0; };
}
// combineFilters helper: combine multiple filters to one
export function combineFilters() {
    var filters = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        filters[_i] = arguments[_i];
    }
    return filters.reduce(function (prev, curr) {
        return function (action, currentState, previousHistory) {
            return prev(action, currentState, previousHistory) &&
                curr(action, currentState, previousHistory);
        };
    }, function () { return true; });
}
export function groupByActionTypes(rawActions) {
    var actions = parseActions(rawActions);
    return function (action) { return actions.indexOf(action.type) >= 0 ? action.type : null; };
}
export function newHistory(past, present, future, group) {
    if (group === void 0) { group = null; }
    return {
        past: past,
        present: present,
        future: future,
        group: group,
        _latestUnfiltered: present,
        index: past.length,
        limit: past.length + future.length + 1
    };
}
//# sourceMappingURL=helpers.js.map